# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_NoteRhythmMaker___call___01():

    maker = rhythmmakertools.NoteRhythmMaker()

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)

    r'''
    \new Staff {
        {
            \time 5/16
            c'4 ~
            c'16
        }
        {
            \time 3/8
            c'4.
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                c'4 ~
                c'16
            }
            {
                \time 3/8
                c'4.
            }
        }
        '''
        )


def test_rhythmmakertools_NoteRhythmMaker___call___02():

    maker = rhythmmakertools.NoteRhythmMaker(decrease_durations_monotonically=False)

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)

    r'''
    \new Staff {
        {
            \time 5/16
            c'16 ~
            c'4
        }
        {
            \time 3/8
            c'4.
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                c'16 ~
                c'4
            }
            {
                \time 3/8
                c'4.
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_01():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 4), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{@ 7:6 c'16, c'8, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_02():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 1, 2, 4), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{@ 4:3 c'16, c'16, c'8, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_03():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(-2, 3, 7),
        mathtools.NonreducedFraction(7, 16),
        )

    assert str(tuplet) == "{@ 12:7 r8, c'8., c'4.. @}"

def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_04():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(7, 7, -4, -1), 
        mathtools.NonreducedFraction(1, 4),
        )

    assert str(tuplet) == "{@ 19:16 c'16.., c'16.., r16, r64 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_05():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 2), 
        mathtools.NonreducedFraction(12, 16),
        )

    assert str(tuplet) == "{@ 5:6 c'8, c'4, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_06():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(2, 4, 4), 
        mathtools.NonreducedFraction(12, 16),
        )

    assert str(tuplet) == "{@ 5:6 c'8, c'4, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_07():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(4, 8, 8), 
        mathtools.NonreducedFraction(12, 16),
        )

    assert str(tuplet) == "{@ 5:3 c'4, c'2, c'2 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_08():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(8, 16, 16), 
        mathtools.NonreducedFraction(12, 16),
        )

    assert str(tuplet) == "{@ 5:3 c'4, c'2, c'2 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_09():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(2, 4, 4), 
        mathtools.NonreducedFraction(3, 16),
        )

    assert str(tuplet) == "{@ 5:3 c'16, c'8, c'8 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_10():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(2, 4, 4), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{@ 5:3 c'8, c'4, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_11():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(2, 4, 4), 
        mathtools.NonreducedFraction(12, 16),
        )

    assert str(tuplet) == "{@ 5:6 c'8, c'4, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_12():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(2, 4, 4), 
        mathtools.NonreducedFraction(24, 16),
        )

    assert str(tuplet) == "{@ 5:6 c'4, c'2, c'2 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_13():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 2), 
        mathtools.NonreducedFraction(6, 2),
        )

    assert str(tuplet) == "{@ 5:6 c'2, c'1, c'1 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_14():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 2), 
        mathtools.NonreducedFraction(6, 4),
        )

    assert str(tuplet) == "{@ 5:6 c'4, c'2, c'2 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_15():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 2), 
        mathtools.NonreducedFraction(6, 8),
        )

    assert str(tuplet) == "{@ 5:6 c'8, c'4, c'4 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_16():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 2, 2), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{@ 5:6 c'16, c'8, c'8 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_17():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, -1, -1), 
        mathtools.NonreducedFraction(3, 16),
        )

    assert str(tuplet) == "{@ 1:1 c'16, r16, r16 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_18():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 1, -1, -1), 
        mathtools.NonreducedFraction(4, 16),
        )

    assert str(tuplet) == "{@ 1:1 c'16, c'16, r16, r16 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_19():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 1, 1, -1, -1), 
        mathtools.NonreducedFraction(5, 16),
        )

    assert str(tuplet) == "{@ 1:1 c'16, c'16, c'16, r16, r16 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_20():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1, 1, 1, 1, -1, -1), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{@ 1:1 c'16, c'16, c'16, c'16, r16, r16 @}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_21():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(1), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{c'4.}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_22():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(99), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{c'4.}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_23():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(-1), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{r4.}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_24():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio(-99), 
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "{r4.}"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_25():

    statement = 'Tuplet.from_nonreduced_ratio_and_nonreduced_fraction'
    statement += '([], (3, 16))'
    pytest.raises(Exception, statement)


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_26():

    statement = 'Tuplet.from_nonreduced_ratio_and_nonreduced_fraction'
    statement += '([0], (3, 16))'
    pytest.raises(Exception, statement)


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_27():

    statement = 'Tuplet.from_nonreduced_ratio_and_nonreduced_fraction'
    statement += '([1, 1, 0, 1], (3, 16))'
    pytest.raises(Exception, statement)



# -*- encoding: utf-8 -*-


def integer_to_binary_string(n):
    r'''Positive integer `n` to binary string.

    ::

        >>> for n in range(1, 16 + 1):
        ...     print '{}\t{}'.format(n, mathtools.integer_to_binary_string(n))
        ...
        1  1
        2  10
        3  11
        4  100
        5  101
        6  110
        7  111
        8  1000
        9  1001
        10 1010
        11 1011
        12 1100
        13 1101
        14 1110
        15 1111
        16 10000

    Returns string.
    '''

    if n == 0:
        return '0'

    result = bin(abs(n)).lstrip('-0b')

    if n < 0:
        result = '-' + result

    return result



# -*- encoding: utf-8 -*-
from abjad.tools.sequencetools.permute_sequence import permute_sequence
import itertools


def yield_all_permutations_of_sequence(sequence):
    '''Yield all permutations of `sequence` in lex order:

    ::

        >>> list(sequencetools.yield_all_permutations_of_sequence((1, 2, 3)))
        [(1, 2, 3), (1, 3, 2), (2, 1, 3), (2, 3, 1), (3, 1, 2), (3, 2, 1)]

    Returns generator of `sequence` objects.
    '''

    for permutation in itertools.permutations(range(len(sequence))):
        yield permute_sequence(sequence, permutation)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class Infinity(AbjadObject):
    r'''Object-oriented infinity.

    All numbers compare less than infinity:

    ::

        >>> 9999999 < Infinity
        True

    ::

        >>> 2**38 < Infinity
        True

    Infinity compares equal to itself:

    ::

        >>> Infinity == Infinity
        True

    Negative infinity compares less than infinity:

    ::

        >>> NegativeInfinity < Infinity
        True

    Infinity is initialized at start-up and is available in the 
    global Abjad namespace.
    '''

    ### INTIALIZER ###

    def __init__(self):
        self._value = float('infinity')

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is also infinity. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._value == expr._value
        return False

    def __ge__(self, expr):
        r''' True for all values of `expr`. Otherwise false.

        Returns boolean.
        '''
        return self._value >= expr

    def __gt__(self, expr):
        r'''True for all noninfinite values of `expr`. Otherwise false.

        Returns boolean.
        '''
        return self._value > expr

    def __le__(self, expr):
        r'''True when `expr` is infinite. Otherwise false.

        Returns boolean.
        '''
        return self._value <= expr

    def __lt__(self, expr):
        r'''True for no values of `expr`.

        Returns boolean.
        '''
        return self._value < expr

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            storage_format_pieces=(
                type(self).__name__,
                ),
            )



# -*- encoding: utf-8 -*-
import copy
import inspect
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import stringtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Instrument(AbjadObject):
    '''A musical instrument.
    '''

    ### CLASS VARIABLES ###

    _format_slot = 'opening'

    __slots__ = (
        '_allowable_clefs',
        '_default_scope',
        '_instrument_name',
        '_instrument_name_markup',
        '_is_primary_instrument',
        '_performer_names',
        '_pitch_range',
        '_short_instrument_name',
        '_short_instrument_name_markup',
        '_sounding_pitch_of_written_middle_c',
        '_starting_clefs',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name=None,
        short_instrument_name=None,
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range=None,
        sounding_pitch_of_written_middle_c=None,
        ):
        from abjad.tools import scoretools
        self._instrument_name = instrument_name
        self._instrument_name_markup = instrument_name_markup
        self._short_instrument_name = short_instrument_name
        self._short_instrument_name_markup = short_instrument_name_markup
        allowable_clefs = allowable_clefs or ['treble']
        allowable_clefs = indicatortools.ClefInventory(allowable_clefs)
        self._allowable_clefs = allowable_clefs
        pitch_range = pitch_range or pitchtools.PitchRange('[A0, C8]')
        pitch_range = pitchtools.PitchRange(pitch_range)
        self._pitch_range = pitch_range
        sounding_pitch_of_written_middle_c = \
            sounding_pitch_of_written_middle_c or pitchtools.NamedPitch("c'")
        sounding_pitch_of_written_middle_c = \
            pitchtools.NamedPitch(sounding_pitch_of_written_middle_c)
        self._sounding_pitch_of_written_middle_c = \
            sounding_pitch_of_written_middle_c
        self._default_scope = scoretools.Staff
        self._is_primary_instrument = False
        self._performer_names = ['instrumentalist']
        self._starting_clefs = copy.copy(allowable_clefs)

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies instrument.

        Returns new instrument.
        '''
        return type(self)(
            instrument_name=self.instrument_name,
            short_instrument_name=self.short_instrument_name,
            instrument_name_markup=self.instrument_name_markup,
            short_instrument_name_markup=self.short_instrument_name_markup,
            allowable_clefs=self.allowable_clefs,
            pitch_range=self.pitch_range,
            sounding_pitch_of_written_middle_c=\
                self.sounding_pitch_of_written_middle_c,
            )

    def __eq__(self, arg):
        r'''Is true when `arg` is an instrument with instrument name and short
        instrument name equal to those of this instrument. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.instrument_name == arg.instrument_name and \
                self.short_instrument_name == arg.short_instrument_name:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats instrument.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        '''Gets hash value instrument.

        Computed on type, instrument name and short instrument name.

        Returns integer.
        '''
        return hash((
            type(self).__name__,
            self.instrument_name, 
            self.short_instrument_name,
            ))

    def __makenew__(
        self,
        instrument_name=None,
        short_instrument_name=None,
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range=None,
        sounding_pitch_of_written_middle_c=None,
        ):
        r'''Makes new instrument.

        Returns new instrument.
        '''
        instrument_name = instrument_name or self.instrument_name
        short_instrument_name = short_instrument_name or \
            self._short_instrument_name
        instrument_name_markup = instrument_name_markup or \
            self.instrument_name_markup
        short_instrument_name_markup = short_instrument_name_markup or \
            self.short_instrument_name_markup
        allowable_clefs = allowable_clefs or self.allowable_clefs
        pitch_range = pitch_range or self.pitch_range
        sounding_pitch_of_written_middle_c = \
            sounding_pitch_of_written_middle_c or \
            self.sounding_pitch_of_written_middle_c
        new = type(self)(
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        return new

    def __repr__(self):
        r'''Gets interpreter representation of instrument.

        Returns string.
        '''
        return '{}()'.format(type(self).__name__)

    ### PRIVATE PROPERTIES ###

    @staticmethod
    def _default_instrument_name_to_instrument_class(default_instrument_name):
        for instrument_class in Instrument._list_instruments():
            instrument = instrument_class()
            if instrument.instrument_name == default_instrument_name:
                return instrument_class

    # TODO: _scope_name needs to be taken from IndicatorExpression!
    #       should not be stored on instrument.
    @property
    def _lilypond_format(self):
        result = []
        line = r'\set {!s}.instrumentName = {!s}'
        line = line.format(
            self._scope_name, 
            self.instrument_name_markup,
            )
        result.append(line)
        line = r'\set {!s}.shortInstrumentName = {!s}'
        line = line.format(
            self._scope_name, 
            self.short_instrument_name_markup,
            )
        result.append(line)
        return result

    @property
    def _one_line_menuing_summary(self):
        return self.instrument_name

    @property
    def _scope_name(self):
        if isinstance(self._default_scope, type):
            return self._default_scope.__name__
        else:
            return type(self._default_scope).__name__

    ### PRIVATE METHODS ###

    def _get_default_performer_name(self):
        if self._performer_names is None:
            performer_name = '{} player'.format(self._default_instrument_name)
            return performer_name
        else:
            return self._performer_names[-1]

    def _get_performer_names(self):
        if self._performer_names is None:
            performer_name = '{} player'.format(self._default_instrument_name)
            return [performer_name]
        else:
            return self._performer_names[:]

    def _initialize_default_name_markups(self):
        string = self.instrument_name
        string = stringtools.capitalize_string_start(string)
        markup = markuptools.Markup(string)
        self._instrument_name_markup = markup
        string = self.short_instrument_name
        string = stringtools.capitalize_string_start(string)
        markup = markuptools.Markup(string)
        self._short_instrument_name_markup = markup

    @classmethod
    def _list_instrument_names(cls):
        r'''Lists instrument names.

        ::

            >>> function = instrumenttools.Instrument._list_instrument_names
            >>> for instrument_name in function():
            ...     instrument_name
            ...
            'accordion'
            'alto'
            'alto flute'
            ...

        Returns list.
        '''
        instrument_names = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            assert instrument.instrument_name is not None, repr(instrument)
            instrument_names.append(instrument.instrument_name)
        instrument_names.sort(key=lambda x: x.lower())
        return instrument_names

    @staticmethod
    def _list_instruments(classes=None):
        r'''Lists instruments.

        ::

            >>> function = instrumenttools.Instrument._list_instruments
            >>> for instrument in function():
            ...     instrument.__name__
            ...
            'Accordion'
            'AltoFlute'
            'AltoSaxophone'
            'AltoTrombone'
            'AltoVoice'
            'BaritoneSaxophone'
            ...

        Returns list.
        '''
        from abjad.tools import instrumenttools
        if classes is None:
            classes = (instrumenttools.Instrument,)
        instruments = []
        for value in instrumenttools.__dict__.itervalues():
            try:
                if issubclass(value, classes):
                    if not value is instrumenttools.Instrument:
                        instruments.append(value)
            except TypeError:
                pass
        instruments.sort(key=lambda x: x.__name__.lower())
        return instruments

    @classmethod
    def _list_primary_instruments(cls):
        primary_instruments = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            if instrument._is_primary_instrument:
                primary_instruments.append(instrument_class)
        return primary_instruments

    @classmethod
    def _list_secondary_instruments(cls):
        secondary_instruments = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            if not instrument._is_primary_instrument:
                secondary_instruments.append(instrument_class)
        return secondary_instruments

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets allowable clefs of instrument.

        Returns clef inventory.
        '''
        if self._allowable_clefs is None:
            self._allowable_clefs = indicatortools.ClefInventory('treble')
        return self._allowable_clefs

    @property
    def instrument_name(self):
        r'''Gets instrument name.

        Returns string.
        '''
        return self._instrument_name

    @property
    def instrument_name_markup(self):
        r'''Gets instrument name markup.

        Returns markup.
        '''
        if self._instrument_name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(self._instrument_name_markup, markuptools.Markup):
            markup = markuptools.Markup(self._instrument_name_markup)
            self._instrument_name_markup = markup
        return self._instrument_name_markup

    @property
    def pitch_range(self):
        r'''Gets pitch range of instrument.

        Returns pitch range.
        '''
        if self._pitch_range is None:
            return self._default_pitch_range
        return self._pitch_range

    @property
    def short_instrument_name(self):
        r'''Gets short instrument name.

        Returns string.
        '''
        if self._short_instrument_name is None:
            return self._default_short_instrument_name
        else:
            return self._short_instrument_name

    @property
    def short_instrument_name_markup(self):
        r'''Gets short instrument name markup.

        Returns markup.
        '''
        if self._short_instrument_name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(
            self._short_instrument_name_markup, markuptools.Markup):
            markup = markuptools.Markup(self._short_instrument_name_markup)
            self._short_instrument_name_markup = markup
        return self._short_instrument_name_markup

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of written middle C.

        Returns named pitch.
        '''
        return self._sounding_pitch_of_written_middle_c



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class HorizontalBracketSpanner(Spanner):
    r'''A horizontal bracket spanner.

    ::

        >>> voice = Voice("c'4 d'4 e'4 f'4")
        >>> voice.engraver_consists.append('Horizontal_bracket_engraver')
        >>> spanner = spannertools.HorizontalBracketSpanner()
        >>> attach(spanner, voice[:])
        >>> show(voice) # doctest: +SKIP

    ::

        >>> spanner
        HorizontalBracketSpanner("c'4, d'4, e'4, f'4")

    ..  doctest::

        >>> print format(voice)
        \new Voice \with {
            \consists Horizontal_bracket_engraver
        } {
            c'4 \startGroup
            d'4
            e'4
            f'4 \stopGroup
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startGroup')
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopGroup')
        return result



# -*- encoding: utf-8 -*-
import os
import webbrowser
from abjad.tools import systemtools
from abjad.tools.documentationtools import AbjadAPIGenerator
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class BuildApiScript(DeveloperScript):
    r'''Builds the Abjad APIs.

    ..  shell::

        ajv api --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'api'``.
        '''
        return 'api'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Build the Abjad APIs.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _build_experimental_api(self, format='html', clean=False):

        from abjad import abjad_configuration

        class ExperimentalAPIGenerator(AbjadAPIGenerator):

            _api_title = 'Abjad Experimental API'

            @property
            def docs_api_index_path(self):
                return os.path.join(
                    abjad_configuration.abjad_experimental_directory_path,
                    'docs', 'source', 'index.rst')

            @property
            def path_definitions(self):
                from abjad import abjad_configuration
                return (
                    (
                        os.path.join(
                            abjad_configuration.abjad_experimental_directory_path, 
                            'tools'),
                        os.path.join(
                            abjad_configuration.abjad_experimental_directory_path, 
                            'docs', 'source', 'tools'),
                        'experimental.tools.',
                    ),
                    (
                        os.path.join(
                            abjad_configuration.abjad_experimental_directory_path, 
                            'demos'),
                        os.path.join(
                            abjad_configuration.abjad_experimental_directory_path, 
                            'docs', 'source', 'demos'),
                        'experimental.demos.',
                    ),
                )

            @property
            def root_package(self):
                return 'experimental'

            @property
            def tools_package_path_index(self):
                return 2

        ExperimentalAPIGenerator()(verbose=True)

        # print greeting
        print 'Now building the Experimental {} docs ...'.format(
            format.upper())
        print ''

        # change to docs directory because makefile lives there
        docs_directory = os.path.join(
            abjad_configuration.abjad_experimental_directory_path, 'docs')
        os.chdir(docs_directory)

        # optionally, make clean before building
        if clean:
            print 'Cleaning build directory ...'
            systemtools.IOManager.spawn_subprocess('make clean')

        if format == 'coverage':
            systemtools.IOManager.spawn_subprocess('sphinx-build -b coverage {} {}'.format(
                'source',
                os.path.join('build', 'coverage'),
                ))
        else:
            systemtools.IOManager.spawn_subprocess('make {}'.format(format))

    def _build_mainline_api(self, format='html', clean=False):
        from abjad import abjad_configuration
        AbjadAPIGenerator()(verbose=True)
        # print greeting
        print 'Now building the {} docs ...'.format(format.upper())
        print ''
        # change to docs directory because makefile lives there
        docs_directory = os.path.relpath(os.path.join(
            abjad_configuration.abjad_directory_path, 'docs'))
        os.chdir(docs_directory)
        # optionally, make clean before building
        if clean:
            print 'Cleaning build directory ...'
            systemtools.IOManager.spawn_subprocess('make clean')
        if format == 'coverage':
            systemtools.IOManager.spawn_subprocess('sphinx-build -b coverage {} {}'.format(
                'source',
                os.path.join('build', 'coverage'),
                ))
        else:
            systemtools.IOManager.spawn_subprocess('make {}'.format(format))

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        from abjad import abjad_configuration
        format = args.format
        clean = args.clean
        paths = []
        if args.mainline:
            self._build_mainline_api(format=format, clean=clean)
            paths.append(os.path.abspath(os.path.join(
                abjad_configuration.abjad_directory_path,
                'docs',
                'build',
                'html',
                'api',
                'index.html',
                )))
        if args.experimental:
            self._build_experimental_api(format=format, clean=clean)
            paths.append(os.path.abspath(os.path.join(
                abjad_configuration.abjad_experimental_directory_path,
                'docs',
                'build',
                'html',
                'index.html',
                )))
        if args.format == 'html' and args.openinbrowser:
            for path in paths:
                if path.startswith('/'):
                    path = 'file://' + path
                webbrowser.open(path)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('-M', '--mainline',
            action='store_true',
            help='build the mainline API'
            )
        parser.add_argument('-X', '--experimental',
            action='store_true',
            help='build the experimental API'
            )
        parser.add_argument('-C', '--clean',
            action='store_true',
            dest='clean',
            help='run "make clean" before building the api',
            )
        parser.add_argument('-O', '--open',
            action='store_true',
            dest='openinbrowser',
            help='open the docs in a web browser after building',
            )
        parser.add_argument('--format',
            choices=(
                'coverage',
                'html',
                'latex',
                'latexpdf',
                ),
            dest='format',
            help='Sphinx builder to use',
            metavar='FORMAT',
            )
        parser.set_defaults(format='html')



# -*- encoding: utf-8 -*-
from abjad import *


def test_quantizationtools_QGrid___eq___01():

    a = quantizationtools.QGrid()
    b = quantizationtools.QGrid()

    assert a == b


def test_quantizationtools_QGrid___eq___02():

    a = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1, children=[
            quantizationtools.QGridLeaf(
                preprolated_duration=1, q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.SilentQEvent(100),
                    0.5
                    )
                ])
            ]),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    )
                ]
            )
        )

    b = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1, children=[
            quantizationtools.QGridLeaf(
                preprolated_duration=1, q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.SilentQEvent(100),
                    0.5
                    )
                ])
            ]),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    )
                ]
            )
        )

    a == b


def test_quantizationtools_QGrid___eq___03():

    a = quantizationtools.QGrid()
    b = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1, children=[
            quantizationtools.QGridLeaf(
                preprolated_duration=1, q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.SilentQEvent(100),
                    0.5
                    )
                ])
            ])
        )
    c = quantizationtools.QGrid(
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    )
                ]
            )
        )
    d = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1, children=[
            quantizationtools.QGridLeaf(
                preprolated_duration=1, q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.SilentQEvent(100),
                    0.5
                    )
                ])
            ]),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    )
                ]
            )
        )

    assert a != b
    assert a != c
    assert a != d

    assert b != c
    assert b != d

    assert c != d



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_fracture_01():
    r'''Fracture container spanner to the right of index 1.
    '''

    staff = Staff(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
        {
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )

    beam = Beam()
    attach(beam, staff[:])
    original, left, right = beam.fracture(1, direction=Right)

    assert len(original.components) == 3
    assert original.components[0] is staff[0]
    assert original.components[1] is staff[1]
    assert original.components[2] is staff[2]
    assert len(original.leaves) == 12

    assert len(left.components) == 2
    assert left.components[0] is staff[0]
    assert left.components[1] is staff[1]
    assert len(left.leaves) == 8

    assert len(right.components) == 1
    assert right.components[0] is staff[2]
    assert len(right.leaves) == 4

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                cs'8
                d'8
                ef'8
            }
            {
                e'8
                f'8
                fs'8
                g'8 ]
            }
            {
                af'8 [
                a'8
                bf'8
                b'8 ]
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class InstrumentInventory(TypedList):
    r'''An ordered list of instruments.

    ::

        >>> inventory = instrumenttools.InstrumentInventory([
        ...     instrumenttools.Flute(), 
        ...     instrumenttools.Guitar()
        ...     ])

    Instrument inventories implement list interface and are mutable.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats instrument inventory.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        superclass = super(InstrumentInventory, self)
        return superclass.__format__(format_specification=format_specification)

    def __repr__(self):
        r'''Gets interpreter representation of instrument inventory.

        ::

            >>> inventory
            InstrumentInventory([Flute(), Guitar()])

        Returns string.
        '''
        contents = [repr(x) for x in self]
        contents = ', '.join(contents)
        return '{}([{}])'.format(type(self).__name__, contents)



# -*- encoding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_get_lilypond_version_string_01():
    lilypond_version_string = \
        systemtools.AbjadConfiguration.get_lilypond_version_string()
    assert isinstance(lilypond_version_string, str)
    assert lilypond_version_string.count('.') == 2



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_special_prolation_01():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    measure = Measure((4, 4), Note("c'4") * 4)
    assert measure[0].written_duration == Duration(1, 4)
    assert inspect(measure[0]).get_duration() == Duration(1, 4)


def test_scoretools_Measure_special_prolation_02():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    measure = Measure((4, 4), [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
        Note("c'4")])
    assert measure.select_leaves()[0].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[0]).get_duration() == Duration(1, 4)
    assert measure.select_leaves()[1].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[1]).get_duration() == Duration(1, 6)


def test_scoretools_Measure_special_prolation_03():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    measure = Measure((4, 4), [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), [
            scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
            Note("c'4")]),
        Note("c'4")])
    assert measure.select_leaves()[0].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[0]).get_duration() == Duration(1, 4)
    assert measure.select_leaves()[1].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[1]).get_duration() == Duration(1, 9)


def test_scoretools_Measure_special_prolation_04():
    r'''Measures without power-of-two time signature denominators
    contribute nontrivially to prolation.
    '''

    measure = Measure((4, 5), Note("c'4") * 4)
    assert measure[0].written_duration == Duration(1, 4)
    assert inspect(measure[0]).get_duration() == Duration(1, 5)


def test_scoretools_Measure_special_prolation_05():
    r'''Measures without power-of-two time signature denominators
    contribute nontrivially to prolation.
    '''

    measure = Measure((4, 5), [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
        Note("c'4")])
    assert measure.select_leaves()[0].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[0]).get_duration() == Duration(1, 5)
    assert measure.select_leaves()[1].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[1]).get_duration() == Duration(2, 15)


def test_scoretools_Measure_special_prolation_06():
    r'''Measures without power-of-two time signature denominators
    contribute nontrivially to prolation.
    '''

    measure = Measure((4, 5), [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), [
            scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
            Note("c'4")]),
        Note("c'4")])
    assert measure.select_leaves()[0].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[0]).get_duration() == Duration(1, 5)
    assert measure.select_leaves()[1].written_duration == Duration(1, 4)
    assert inspect(measure.select_leaves()[1]).get_duration() == Duration(4, 45)



# -*- encoding: utf-8 -*-
from AbjadConfiguration import AbjadConfiguration
from BenchmarkScoreMaker import BenchmarkScoreMaker
from Configuration import Configuration
from IOManager import IOManager
from ImportManager import ImportManager
from LilyPondFormatBundle import LilyPondFormatBundle
from LilyPondFormatManager import LilyPondFormatManager
from RedirectedStreams import RedirectedStreams
from StorageFormatManager import StorageFormatManager
from StorageFormatSpecification import StorageFormatSpecification
from TestManager import TestManager
from Timer import Timer
from UpdateManager import UpdateManager
from WellformednessManager import WellformednessManager
from requires import requires

_documentation_section = 'internals'



# -*- encoding: utf-8 -*-
from abjad.tools.rhythmmakertools.DivisionIncisedNoteRhythmMaker \
    import DivisionIncisedNoteRhythmMaker


class NoteRhythmMaker(DivisionIncisedNoteRhythmMaker):
    r'''Note rhythm-maker:

    ..  container:: example

        **Example 1:**

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker()

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> leaf_lists = maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 5/8
                    c'2 ~
                    c'8
                }
                {
                    \time 3/8
                    c'4.
                }
            }

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example
    
        **Example 2.** Forbid half notes:

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     forbidden_written_duration=Duration(1, 2))

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> leaf_lists = maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 5/8
                    c'4 ~
                    c'4 ~
                    c'8
                }
                {
                    \time 3/8
                    c'4.
                }
            }

        ::

            >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        decrease_durations_monotonically=True,
        forbidden_written_duration=None,
        tie_rests=False,
        ):
        DivisionIncisedNoteRhythmMaker.__init__(
            self,
            [],
            [0],
            [],
            [0],
            1,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            tie_rests=tie_rests,
            )

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats note rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.NoteRhythmMaker(
                decrease_durations_monotonically=True,
                forbidden_written_duration=durationtools.Duration(1, 2),
                tie_rests=False,
                )

        Returns string.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new note rhythm-maker.

        ::

            >>> new_maker = new(maker, decrease_durations_monotonically=False)

        ::

            >>> print format(new_maker)
            rhythmmakertools.NoteRhythmMaker(
                decrease_durations_monotonically=False,
                forbidden_written_duration=durationtools.Duration(1, 2),
                tie_rests=False,
                )

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> leaf_lists = new_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new note rhythm-maker.
        '''
        return DivisionIncisedNoteRhythmMaker.__makenew__(
            self, *args, **kwargs)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses note rhythm-maker.

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.NoteRhythmMaker(
                decrease_durations_monotonically=False,
                forbidden_written_duration=durationtools.Duration(1, 2),
                tie_rests=False,
                )

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> leaf_lists = reversed_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new note rhythm-maker.
        '''
        return DivisionIncisedNoteRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.topleveltools import override


def make_time_signature_context_block(
    font_size=3, 
    minimum_distance=12, 
    padding=4,
    ):
    r'''Make time signature context block:

    ::

        >>> context_block = lilypondfiletools.make_time_signature_context_block()

    ..  doctest::

        >>> print format(context_block) # doctest: +SKIP
        \context {
            \type Engraver_group
            \name TimeSignatureContext
            \consists Axis_group_engraver
            \consists Time_signature_engraver
            \override TimeSignature #'X-extent = #'(0 . 0)
            \override TimeSignature #'X-offset = #ly:self-alignment-interface::x-aligned-on-self
            \override TimeSignature #'Y-extent = #'(0 . 0)
            \override TimeSignature #'break-align-symbol = ##f
            \override TimeSignature #'break-visibility = #end-of-line-invisible
            \override TimeSignature #'font-size = #3
            \override TimeSignature #'self-alignment-X = #center
            \override VerticalAxisGroup #'default-staff-staff-spacing = #'(
                (basic-distance . 0) (minimum-distance . 0) (padding . 4) (stretchability . 0))
        }

    Returns context block.
    '''
    from abjad.tools import layouttools
    from abjad.tools import lilypondfiletools

    assert isinstance(font_size, (int, float))
    assert isinstance(padding, (int, float))

    context_block = lilypondfiletools.ContextBlock()
    context_block.type = 'Engraver_group'
    context_block.engraver_consists.append('Axis_group_engraver')
    context_block.engraver_consists.append('Time_signature_engraver')
    context_block.name = 'TimeSignatureContext'
    override(context_block).time_signature.X_extent = (0, 0)
    override(context_block).time_signature.X_offset = schemetools.Scheme(
        'ly:self-alignment-interface::x-aligned-on-self')
    override(context_block).time_signature.Y_extent = (0, 0)
    override(context_block).time_signature.break_align_symbol = False
    override(context_block).time_signature.break_visibility = \
        schemetools.Scheme('end-of-line-invisible')
    override(context_block).time_signature.font_size = font_size
    override(context_block).time_signature.self_alignment_X = \
        schemetools.Scheme('center')
    spacing_vector = layouttools.make_spacing_vector(
        0, 
        minimum_distance, 
        padding, 
        0,
        )
    override(context_block).vertical_axis_group.default_staff_staff_spacing = \
        spacing_vector

    return context_block



# -*- encoding: utf-8 -*-
import pytest
from abjad.tools import stringtools


def test_arg_to_bidirectional_lilypond_symbol_01():

    assert stringtools.arg_to_bidirectional_lilypond_symbol('^') == '^'
    assert stringtools.arg_to_bidirectional_lilypond_symbol('_') == '_'
    assert stringtools.arg_to_bidirectional_lilypond_symbol(Up) == '^'
    assert stringtools.arg_to_bidirectional_lilypond_symbol(Down) == '_'
    assert stringtools.arg_to_bidirectional_lilypond_symbol(1) == '^'
    assert stringtools.arg_to_bidirectional_lilypond_symbol(-1) == '_'
    assert pytest.raises(ValueError,
        "stringtools.arg_to_bidirectional_lilypond_symbol('foo')")



# -*- encoding: utf-8 -*-
import abc
import copy
import types
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate


class BurnishedRhythmMaker(RhythmMaker):
    '''Abstract base class for rhythm-makers that burnish some or
    all of the output cells they produce.

    'Burnishing' means to forcibly cast the first or last
    (or both first and last) elements of a output cell to be
    either a note or rest.

    'Division-burnishing' rhythm-makers burnish every output cell they 
    produce.

    'Output-burnishing' rhythm-makers burnish only the first and last
    output cells they produce and leave interior output cells unchanged.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [-1, 4, -2, 3], 
        16,
        )

    ### INITIALIZER ###

    def __init__(self, 
        talea=None, 
        talea_denominator=None, 
        prolation_addenda=None,
        lefts=None, 
        middles=None, 
        rights=None, 
        left_lengths=None, 
        right_lengths=None,
        secondary_divisions=None,
        talea_helper=None, 
        prolation_addenda_helper=None,
        lefts_helper=None, 
        middles_helper=None, 
        rights_helper=None,
        left_lengths_helper=None, 
        right_lengths_helper=None, 
        secondary_divisions_helper=None,
        beam_each_cell=False, 
        beam_cells_together=False,
        decrease_durations_monotonically=True, 
        tie_split_notes=False, 
        tie_rests=False,
        ):
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together,
            )
        prolation_addenda = self._none_to_new_list(prolation_addenda)
        lefts = self._none_to_new_list(lefts)
        middles = self._none_to_new_list(middles)
        rights = self._none_to_new_list(rights)
        left_lengths = self._none_to_new_list(left_lengths)
        right_lengths = self._none_to_new_list(right_lengths)
        secondary_divisions = self._none_to_new_list(secondary_divisions)
        talea_helper = self._none_to_trivial_helper(talea_helper)
        prolation_addenda_helper = self._none_to_trivial_helper(
            prolation_addenda_helper)
        lefts_helper = self._none_to_trivial_helper(lefts_helper)
        middles_helper = self._none_to_trivial_helper(middles_helper)
        rights_helper = self._none_to_trivial_helper(rights_helper)
        left_lengths_helper = self._none_to_trivial_helper(
            left_lengths_helper)
        right_lengths_helper = self._none_to_trivial_helper(
            right_lengths_helper)
        secondary_divisions_helper = self._none_to_trivial_helper(
            secondary_divisions_helper)
        assert sequencetools.all_are_integer_equivalent_numbers(talea)
        assert mathtools.is_positive_integer_equivalent_number(
            talea_denominator)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            prolation_addenda)
        assert all(x in (-1, 0, 1) for x in lefts)
        assert all(x in (-1, 0, 1) for x in middles)
        assert all(x in (-1, 0, 1) for x in rights)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            left_lengths)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            right_lengths)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            secondary_divisions)
        assert isinstance(talea_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(prolation_addenda_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(lefts_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(middles_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(rights_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(left_lengths_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(right_lengths_helper, 
            (types.FunctionType, types.MethodType))
        assert isinstance(decrease_durations_monotonically, bool)
        assert isinstance(tie_split_notes, bool)
        assert isinstance(tie_rests, bool)
        self.talea = talea
        self.talea_denominator = talea_denominator
        self.prolation_addenda = prolation_addenda
        self.lefts = lefts
        self.middles = middles
        self.rights = rights
        self.left_lengths = left_lengths
        self.right_lengths = right_lengths
        self.secondary_divisions = secondary_divisions
        self.talea_helper = talea_helper
        self.prolation_addenda_helper = prolation_addenda_helper
        self.lefts_helper = lefts_helper
        self.middles_helper = middles_helper
        self.rights_helper = rights_helper
        self.left_lengths_helper = left_lengths_helper
        self.right_lengths_helper = right_lengths_helper
        self.secondary_divisions_helper = secondary_divisions_helper
        #self.beam_each_cell = beam_each_cell
        self.decrease_durations_monotonically = \
            decrease_durations_monotonically
        self.tie_split_notes = tie_split_notes
        self.tie_rests = tie_rests

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls burnished rhythm-maker on `divisions`.

        Returns either list of tuplets or else list of note-lists.
        '''
        duration_pairs, seeds = RhythmMaker.__call__(self, divisions, seeds)
        octuplet = self._prepare_input(seeds)
        talea, prolation_addenda = octuplet[:2]
        secondary_divisions = octuplet[-1]
        talee = (talea, prolation_addenda, secondary_divisions)
        result = self._scale_talee(
            duration_pairs, self.talea_denominator, talee)
        duration_pairs, lcd, talea, prolation_addenda, secondary_divisions = \
            result
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, secondary_divisions)
        septuplet = (talea, prolation_addenda) + octuplet[2:-1]
        numeric_map = self._make_numeric_map(
            secondary_duration_pairs, septuplet)
        leaf_lists = self._make_leaf_lists(numeric_map, lcd)
        if not prolation_addenda:
            result = leaf_lists
        else:
            tuplets = self._make_tuplets(secondary_duration_pairs, leaf_lists)
            result = tuplets
        if self.beam_each_cell:
            for cell in result:
                beam = spannertools.MultipartBeam()
                attach(beam, cell)
        if self.tie_split_notes:
            self._add_ties(result)
        assert isinstance(result, list), repr(result)
        assert all(isinstance(x, selectiontools.Selection) for x in result) or \
            all(isinstance(x, scoretools.Tuplet) for x in result)
        return result

    def __format__(self, format_specification=''):
        r'''Formats burnished rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        Returns string.
        '''
        superclass = super(BurnishedRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _add_ties(self, result):
        from abjad.tools import selectiontools
        leaves = list(iterate(result).by_class(scoretools.Leaf))
        written_durations = [leaf.written_duration for leaf in leaves]
        weights = [durationtools.Duration(numerator, self.talea_denominator) 
            for numerator in self.talea]
        parts = sequencetools.partition_sequence_by_weights_exactly(
            written_durations, weights=weights, cyclic=True, overhang=True)
        counts = [len(part) for part in parts]
        parts = sequencetools.partition_sequence_by_counts(leaves, counts)
        prototype = (spannertools.Tie,)
        for part in parts:
            part = selectiontools.SliceSelection(part)
            tie_spanner = spannertools.Tie()
            # this is voodoo to temporarily neuter the contiguity constraint
            tie_spanner._contiguity_constraint = None
            for component in part:
                # TODO: make top-level detach() work here
                for spanner in component._get_spanners(
                    prototype=prototype):
                    spanner.detach()
                #detach(prototype, component)
            tie_spanner.extend(part)

    def _burnish_division_part(self, division_part, token):
        assert len(division_part) == len(token)
        new_division_part = []
        for number, i in zip(division_part, token):
            if i == -1:
                new_division_part.append(-abs(number))
            elif i == 0:
                new_division_part.append(number)
            elif i == 1:
                new_division_part.append(abs(number))
            else:
                raise ValueError
        new_division_part = type(division_part)(new_division_part)
        return new_division_part

    @abc.abstractmethod
    def _burnish_division_parts(self, divisions, quintuplet):
        pass

    def _make_leaf_lists(self, numeric_map, talea_denominator):
        leaf_lists = []
        for map_division in numeric_map:
            leaf_list = scoretools.make_leaves_from_talea(
                map_division, talea_denominator,
                decrease_durations_monotonically=self.decrease_durations_monotonically,
                tie_rests=self.tie_rests)
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _make_numeric_map(self, duration_pairs, septuplet):
        talea, prolation_addenda, lefts, middles, rights, left_lengths, right_lengths = septuplet
        prolated_duration_pairs = self._make_prolated_duration_pairs(
            duration_pairs, prolation_addenda)
        if isinstance(prolated_duration_pairs[0], tuple):
            prolated_numerators = [
                pair[0] for pair in prolated_duration_pairs]
        else:
            prolated_numerators = [
                pair.numerator for pair in prolated_duration_pairs]
        map_divisions = sequencetools.split_sequence_extended_to_weights(
            talea, prolated_numerators, overhang=False)
        quintuplet = (lefts, middles, rights, left_lengths, right_lengths)
        burnished_map_divisions = self._burnish_division_parts(
            map_divisions, quintuplet)
        numeric_map = burnished_map_divisions
        return numeric_map

    def _make_prolated_duration_pairs(self, duration_pairs, prolation_addenda):
        prolated_duration_pairs = []
        for i, duration_pair in enumerate(duration_pairs):
            if not prolation_addenda:
                prolated_duration_pairs.append(duration_pair)
            else:
                prolation_addendum = prolation_addenda[i]
                if hasattr(duration_pair, 'numerator'):
                    prolation_addendum %= duration_pair.numerator
                else:
                    prolation_addendum %= duration_pair[0]
                if isinstance(duration_pair, tuple):
                    numerator, denominator = duration_pair
                else:
                    numerator, denominator = duration_pair.pair
                prolated_duration_pair = (
                    numerator + prolation_addendum, denominator)
                prolated_duration_pairs.append(prolated_duration_pair)
        return prolated_duration_pairs

    def _prepare_input(self, seeds):
        talea = datastructuretools.CyclicTuple(self.talea_helper(self.talea, seeds))
        prolation_addenda = \
            self.prolation_addenda_helper(self.prolation_addenda, seeds)
        prolation_addenda = datastructuretools.CyclicTuple(prolation_addenda)
        lefts = datastructuretools.CyclicTuple(self.lefts_helper(self.lefts, seeds))
        middles = datastructuretools.CyclicTuple(
            self.middles_helper(self.middles, seeds))
        rights = datastructuretools.CyclicTuple(
            self.rights_helper(self.rights, seeds))
        left_lengths = datastructuretools.CyclicTuple(
            self.left_lengths_helper(self.left_lengths, seeds))
        right_lengths = datastructuretools.CyclicTuple(
            self.right_lengths_helper(self.right_lengths, seeds))
        secondary_divisions = self.secondary_divisions_helper(
            self.secondary_divisions, seeds)
        secondary_divisions = datastructuretools.CyclicTuple(
            secondary_divisions)
        return (
            talea,
            prolation_addenda,
            lefts, 
            middles, 
            rights, 
            left_lengths, 
            right_lengths, 
            secondary_divisions,
            )

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses burnished rhythm-maker.

        Defined equal to a copy of rhythm-maker with all the following
        lists reversed:

        ::

            new.talea
            new.prolation_addenda
            new.lefts
            new.middles
            new.rights
            new.left_lengths
            new.right_lengths
            new.secondary_divisions

        Returns newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        new.talea.reverse()
        new.prolation_addenda.reverse()
        new.lefts.reverse()
        new.middles.reverse()
        new.rights.reverse()
        new.left_lengths.reverse()
        new.right_lengths.reverse()
        new.secondary_divisions.reverse()
        if new.decrease_durations_monotonically:
            new.decrease_durations_monotonically = False
        return new



# -*- encoding: utf-8 -*-
import types
from abjad.tools.documentationtools.Documenter import Documenter


class FunctionDocumenter(Documenter):
    r'''FunctionDocumenter generates an ReST entry for a given function:

    ::

        >>> documenter = documentationtools.FunctionDocumenter(scoretools.make_notes)
        >>> print documenter()
        scoretools.make_notes
        =====================
        <BLANKLINE>
        .. autofunction:: abjad.tools.scoretools.make_notes.make_notes
        <BLANKLINE>

    Returns ``FunctionDocumenter``` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, object_=None, prefix='abjad.tools.'):
        if isinstance(object_, types.FunctionType):
            if object_.__name__ == 'wrapper':
                object_ = object_.func_closure[1].cell_contents
        Documenter.__init__(self, object_, prefix)

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Generate documentation.

        Returns string.
        '''
        from abjad.tools import documentationtools
        document = documentationtools.ReSTDocument()
        stripped_function_name = self._shrink_module_name(
            self.object_.__module__)
        parts = self.object_.__module__.split('.')
        tools_package_path = '.'.join(parts[:3])
        tools_package_name, sep, function_name = \
            stripped_function_name.partition('.')
        banner = '{}.{}'.format(tools_package_name, function_name)
#        banner = ':py:mod:`{} <{}>`.{}'.format(
#            tools_package_name,
#            tools_package_path,
#            function_name,
#            )
        heading = documentationtools.ReSTHeading(
            level=2,
            text=banner,
            )
        autodoc = documentationtools.ReSTAutodocDirective(
            argument=self.module_name,
            directive='autofunction',
            options={
                },
            )
        document.extend([heading, autodoc])
        return document.rest_format



# -*- encoding: utf-8 -*-


def cumulative_signed_weights(sequence):
    r'''Cumulative signed weights of `sequence`.

    ::

        >>> l = [1, -2, -3, 4, -5, -6, 7, -8, -9, 10]
        >>> mathtools.cumulative_signed_weights(l)
        [1, -3, -6, 10, -15, -21, 28, -36, -45, 55]

    Raises type error when `sequence` is not a list.

    Use ``mathtools.cumulative_sums([abs(x) for x in l])``
    for cumulative (unsigned) weights 

    Returns list.
    '''
    from abjad.tools import mathtools

    if not isinstance(sequence, list):
        raise TypeError

    result = []

    for x in sequence:
        try:
            next_element = abs(previous) + abs(x)
            previous_sign = mathtools.sign(previous)
        except NameError:
            next_element = abs(x)
            previous_sign = 0
        sign_x = mathtools.sign(x)
        if sign_x == -1:
            next_element *= sign_x
        elif sign_x == 0:
            next_element *= previous_sign
        result.append(next_element)
        previous = next_element

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_logical_ties_in_expr_with_logical_tie_durations_01():

    staff = Staff(scoretools.make_repeated_notes(4))
    scoretools.FixedDurationTuplet(Duration(2, 8), staff[:3])
    tie = spannertools.Tie()
    attach(tie, staff.select_leaves()[:2])
    tie = spannertools.Tie()
    attach(tie, staff.select_leaves()[2:])
    labeltools.label_logical_ties_in_expr_with_logical_tie_durations(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \times 2/3 {
                c'8 ~
                    _ \markup {
                        \column
                            {
                                \small
                                    1/4
                                \small
                                    1/6
                            }
                        }
                c'8
                c'8 ~
                    _ \markup {
                        \column
                            {
                                \small
                                    1/4
                                \small
                                    5/24
                            }
                        }
            }
            c'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad import *


def test_instrumenttools_transpose_from_written_pitch_to_sounding_pitch_01():

    staff = Staff("<c' e' g'>4 d'4 r4 e'4")
    clarinet = instrumenttools.ClarinetInBFlat()
    attach(clarinet, staff)
    instrumenttools.transpose_from_written_pitch_to_sounding_pitch(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Clarinet in B-flat }
            \set Staff.shortInstrumentName = \markup { Cl. in B-flat }
            <bf d' f'>4
            c'4
            r4
            d'4
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def partition_sequence_by_ratio_of_lengths(sequence, lengths):
    '''Partition `sequence` by ratio of `lengths`:

    ::

        >>> sequence = tuple(range(10))

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_lengths(
        ...     sequence, 
        ...     [1, 1, 2],
        ...     )
        [(0, 1, 2), (3, 4), (5, 6, 7, 8, 9)]

    Use rounding magic to avoid fractional part lengths.

    Returns list of `sequence` objects.
    '''
    from abjad.tools import sequencetools

    lengths = mathtools.partition_integer_by_ratio(len(sequence), lengths)
    return sequencetools.partition_sequence_by_counts(
        sequence, 
        lengths, 
        cyclic=False, 
        overhang=False,
        )



# -*- encoding: utf-8 -*-
import fractions
import numbers
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import sequencetools


def make_notes(pitches, durations, decrease_durations_monotonically=True):
    r'''Make notes according to `pitches` and `durations`.

    Cycle through `pitches` when the length of `pitches` is less than the
    length of `durations`:

    ::

        >>> scoretools.make_notes([0], [(1, 16), (1, 8), (1, 8)])
        Selection(Note("c'16"), Note("c'8"), Note("c'8"))

    Cycle through `durations` when the length of `durations` is less than the
    length of `pitches`:

    ::

        >>> scoretools.make_notes([0, 2, 4, 5, 7], [(1, 16), (1, 8), (1, 8)])
        Selection(Note("c'16"), Note("d'8"), Note("e'8"), Note("f'16"), Note("g'8"))

    Create ad hoc tuplets for nonassignable durations:

    ::

        >>> scoretools.make_notes([0], [(1, 16), (1, 12), (1, 8)])
        Selection(Note("c'16"), Tuplet(Multiplier(2, 3), "c'8"), Note("c'8"))

    Set ``decrease_durations_monotonically=True`` to express tied values 
    in decreasing duration:

    ::

        >>> scoretools.make_notes(
        ...     [0], 
        ...     [(13, 16)], 
        ...     decrease_durations_monotonically=True,
        ...     )
        Selection(Note("c'2."), Note("c'16"))

    Set ``decrease_durations_monotonically=False`` to express tied 
    values in increasing duration:

    ::

        >>> scoretools.make_notes(
        ...     [0], 
        ...     [(13, 16)], 
        ...     decrease_durations_monotonically=False,
        ...     )
        Selection(Note("c'16"), Note("c'2."))

    Set `pitches` to a single pitch or a sequence of pitches.

    Set `durations` to a single duration or a list of durations.

    Returns list of newly constructed notes.
    '''
    from abjad.tools import scoretools
    from abjad.tools import scoretools
    from abjad.tools import selectiontools
    from abjad.tools import scoretools

    if isinstance(pitches, str):
        pitches = pitches.split()

    if not isinstance(pitches, list):
        pitches = [pitches]

    if isinstance(durations, (numbers.Number, tuple)):
        durations = [durations]

    duration_pairs = [durationtools.Duration(duration) 
        for duration in durations]

    # set lists of pitches and duration pairs to the same length
    size = max(len(duration_pairs), len(pitches))
    duration_pairs = \
        sequencetools.repeat_sequence_to_length(duration_pairs, size)
    pitches = sequencetools.repeat_sequence_to_length(pitches, size)

    Duration = durationtools.Duration
    durations = Duration._group_nonreduced_fractions_by_implied_prolation(
        duration_pairs)

    def _make_unprolated_notes(
        pitches,
        durations,
        decrease_durations_monotonically=decrease_durations_monotonically,
        ):
        assert len(pitches) == len(durations)
        result = []
        for pitch, duration in zip(pitches, durations):
            result.extend(scoretools.make_tied_leaf(
                scoretools.Note,
                duration,
                pitches=pitch,
                decrease_durations_monotonically=decrease_durations_monotonically,
                ))
        return result

    result = []
    for duration in durations:
        # get factors in denominator of duration group duration other than 1, 2.
        factors = set(mathtools.factors(duration[0].denominator))
        factors.discard(1)
        factors.discard(2)
        ps = pitches[0:len(duration)]
        pitches = pitches[len(duration):]
        if len(factors) == 0:
            result.extend(_make_unprolated_notes(ps, duration,
                decrease_durations_monotonically=decrease_durations_monotonically))
        else:
            # compute prolation
            denominator = duration[0].denominator
            numerator = mathtools.greatest_power_of_two_less_equal(denominator)
            multiplier = (numerator, denominator)
            ratio = 1 / fractions.Fraction(*multiplier)
            duration = [ratio * durationtools.Duration(d) for d in duration]
            ns = _make_unprolated_notes(ps, duration,
                decrease_durations_monotonically=decrease_durations_monotonically)
            t = scoretools.Tuplet(multiplier, ns)
            result.append(t)

    # return result
    result = selectiontools.Selection(result)
    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_TempoInventory_append_01():

    tempo_inventory_1 = indicatortools.TempoInventory([((1, 8), 72)])
    tempo_inventory_1.append(((1, 8), 84))
    tempo_inventory_2 = indicatortools.TempoInventory([
        ((1, 8), 72), 
        ((1, 8), 84),
        ])

    assert tempo_inventory_1 == tempo_inventory_2



# -*- encoding: utf-8 -*-


def iterate_sequence_pairwise_strict(sequence):
    '''Iterate `sequence` pairwise strict:

    ::

        >>> list(sequencetools.iterate_sequence_pairwise_strict(range(6)))
        [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5)]

    Returns pair generator.
    '''

    previous = None
    for x in sequence:
        current = x
        if previous is not None:
            yield previous, current
        previous = current



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSegment_local_maxima_01():

    pitch_segment = pitchtools.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])
    assert pitch_segment.local_maxima == (NamedPitch(7), )



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import iterate
from abjad.tools.scoretools.Component import Component


class Container(Component):
    r'''An iterable container of music.

    **Example**:

    ..  container:: example

        ::

            >>> container = Container("c'4 e'4 d'4 e'8 f'8")
            >>> show(container) # doctest: +SKIP

        ..  doctest::

            >>> print format(container)
            {
                c'4
                e'4
                d'4
                e'8
                f'8
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_formatter',
        '_music',
        '_named_children',
        '_simultaneous',
        )

    _default_positional_input_arguments = (
        [],
        )

    _storage_format_attribute_mapping = {
        'music': '_music',
        }

    ### INITIALIZER ###

    def __init__(self, music=None):
        Component.__init__(self)
        self._named_children = {}
        self._simultaneous = False
        self._initialize_music(music)

    ### SPECIAL METHODS ###

    def __contains__(self, expr):
        r'''True when `expr` appears in container.
        Otherwise false.

        Returns boolean.
        '''
        for x in self._music:
            if x is expr:
                return True
        else:
            return False

    def __delitem__(self, i):
        r'''Delete container `i`.
        Detach component(s) from parentage.
        Withdraw component(s) from crossing spanners.
        Preserve spanners that component(s) cover(s).

        Returns none.
        '''
        components = self[i]
        #if not isinstance(components, selectiontools.SliceSelection):
        if not isinstance(components, selectiontools.Selection):
            components = selectiontools.SliceSelection([components])
        if not self.is_simultaneous:
            components._withdraw_from_crossing_spanners()
        components._set_parents(None)

    def __getitem__(self, i):
        r'''Gets container `i`.

        Traverses top-level items only.

        Returns component.
        '''
        if isinstance(i, int):
            return self._music[i]
        elif isinstance(i, slice) and not self.is_simultaneous:
            return selectiontools.SliceSelection(self._music[i])
        elif isinstance(i, slice) and self.is_simultaneous:
            return selectiontools.SimultaneousSelection(self._music[i])
        elif isinstance(i, str):
            if i not in self._named_children:
                message = 'can not find component named {!r}.'
                message = message.format(i)
                raise ValueError(message)
            elif 1 < len(self._named_children[i]):
                message = 'multiple components named {!r}.'
                message = message.format(i)
                raise ValueError(message)
            return self._named_children[i][0]
        message = 'can not get container item {!r}.'
        message = message.format(i)
        raise ValueError(message)

    def __len__(self):
        r'''Number of items in container.

        Returns nonnegative integer.
        '''
        return len(self._music)

    def __setitem__(self, i, expr):
        r'''Set container `i` equal to `expr`.
        Find spanners that dominate self[i] and children of self[i].
        Replace contents at self[i] with 'expr'.
        Reattach spanners to new contents.
        This operation always leaves score tree in tact.

        Returns none.
        '''
        return self._set_item(i, expr)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        if not self.is_simultaneous:
            return '{%s}' % self._summary
        else:
            return '<<%s>>' % self._summary

    @property
    def _contents_duration(self):
        if self.is_simultaneous:
            return max([durationtools.Duration(0)] +
                [x._preprolated_duration for x in self])
        else:
            duration = durationtools.Duration(0)
            for x in self:
                duration += x._preprolated_duration
            return duration

    @property
    def _duration_in_seconds(self):
        from abjad.tools import scoretools
        if self.is_simultaneous:
            return max([durationtools.Duration(0)] +
                [x._get_duration(in_seconds=True) for x in self])
        else:
            duration = durationtools.Duration(0)
            for leaf in iterate(self).by_class(scoretools.Leaf):
                duration += leaf._get_duration(in_seconds=True)
            return duration

    @property
    def _preprolated_duration(self):
        return self._contents_duration

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(),
            storage_format_pieces=(
                self._compact_representation,
                ),
            )

    @property
    def _space_delimited_summary(self):
        if 0 < len(self):
            result = []
            for x in self._music:
                if hasattr(x, '_compact_representation_with_tie'):
                    result.append(x._compact_representation_with_tie)
                else:
                    result.append(str(x))
            return ' '.join(result)
        else:
            return ''

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        if not self:
            positional_argument_values = ()
        else:
            lilypond_format = ' '.join(format(x, 'lilypond') for x in self)
            lilypond_format = lilypond_format.replace('\n', ' ')
            lilypond_format = lilypond_format.replace('\t', ' ')
            lilypond_format = lilypond_format.replace('  ', ' ')
            positional_argument_values = (lilypond_format,)
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self._music])
        else:
            return ''

    ### PRIVATE METHODS ###

    def _append_without_withdrawing_from_crossing_spanners(self, component):
        '''Not composer-safe.
        '''
        self._set_item(slice(len(self), len(self)), [component],
            withdraw_components_in_expr_from_crossing_spanners=False)

    def _copy_with_children_and_indicators_but_without_spanners(self):
        new = self._copy_with_indicators_but_without_children_or_spanners()
        for component in self:
            new_component = \
                component._copy_with_children_and_indicators_but_without_spanners()
            new.append(new_component)
        return new

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Component._copy_with_indicators_but_without_children_or_spanners(self)
        new.is_simultaneous = self.is_simultaneous
        return new

    def _format_after_slot(self, bundle):
        result = []
        result.append(('commands', bundle.after.commands))
        result.append(('comments', bundle.after.comments))
        return tuple(result)

    def _format_before_slot(self, bundle):
        result = []
        result.append(('comments', bundle.before.comments))
        result.append(('commands', bundle.before.commands))
        return tuple(result)

    def _format_close_brackets_slot(self, bundle):
        result = []
        if self.is_simultaneous:
            brackets_close = ['>>']
        else:
            brackets_close = ['}']
        result.append([('close brackets', ''), brackets_close])
        return tuple(result)

    def _format_closing_slot(self, bundle):
        result = []
        result.append(('grob reverts', bundle.grob_reverts))
        result.append(('commands', bundle.closing.commands))
        result.append(('comments', bundle.closing.comments))
        return self._format_slot_contributions_with_indent(result)

    def _format_content_pieces(self):
        result = []
        for m in self._music:
            result.extend(format(m).split('\n'))
        result = ['\t' + x for x in result]
        return result

    def _format_contents_slot(self, bundle):
        result = []
        result.append([('contents', '_contents'), self._format_content_pieces()])
        return tuple(result)

    def _format_open_brackets_slot(self, bundle):
        result = []
        if self.is_simultaneous:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        result.append([('open brackets', ''), brackets_open])
        return tuple(result)

    def _format_opening_slot(self, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('commands', bundle.opening.commands))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        return self._format_slot_contributions_with_indent(result)

    def _format_slot_contributions_with_indent(self, slot):
        result = []
        for contributor, contributions in slot:
            result.append(
                (contributor, tuple(['\t' + x for x in contributions])))
        return tuple(result)

    def _get_spanners_that_dominate_component_pair(self, left, right):
        r'''Returns spanners that dominant component pair.
        Returns set (spanner, index) pairs.
        `left` must be an Abjad component or None.
        `right` must be an Abjad component or None.

        If both `left` and `right` are components,
        then `left` and `right` must be logical-voice-contiguous.

        This is a version of ContiguousSelection._get_dominant_spanners().
        This version is useful for finding spanners that dominant
        a zero-length slice between components, as in staff[2:2].
        '''
        from abjad.tools import spannertools
        Selection = selectiontools.Selection
        if left is None or right is None:
            return set([])
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            [left, right])
        left_contained = left._get_descendants()._get_spanners()
        right_contained = right._get_descendants()._get_spanners()
        dominant_spanners = left_contained & right_contained
        right_start_offset = right._get_timespan().start_offset
        components_after_gap = []
        for component in right._get_lineage():
            if component._get_timespan().start_offset == right_start_offset:
                components_after_gap.append(component)
        receipt = set([])
        for spanner in dominant_spanners:
            for component in components_after_gap:
                if component in spanner:
                    index = spanner.index(component)
                    receipt.add((spanner, index))
                    continue
        return receipt

    def _get_spanners_that_dominate_slice(self, start, stop):
        from abjad.tools import spannertools
        if start == stop:
            if start == 0:
                left = None
            else:
                left = self[start - 1]
            if len(self) <= stop:
                right = None
            else:
                right = self[stop]
            spanners_receipt = \
                self._get_spanners_that_dominate_component_pair(left, right)
        else:
            selection = self[start:stop]
            spanners_receipt = selection._get_dominant_spanners()
        return spanners_receipt

    def _scale_contents(self, multiplier):
        for expr in iterate(self[:]).by_topmost_logical_ties_and_components():
            expr._scale(multiplier)

    def _set_item(
        self,
        i,
        expr,
        withdraw_components_in_expr_from_crossing_spanners=True,
        ):
        r'''This method exists beacuse __setitem__ can not accept keywords.
        Note that setting
        withdraw_components_in_expr_from_crossing_spanners=False
        constitutes a composer-unsafe use of this method.
        Only private methods should set this keyword.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import spannertools
        # cache indicators attached to components in expr
        expr_indicators = []
        for component in iterate(expr).by_class():
            indicators = component._get_indicators(unwrap=False)
            expr_indicators.extend(indicators)
        # item assignment
        if isinstance(i, int):
            if isinstance(expr, str):
                expr = self._parse_string(expr)[:]
                assert len(expr) == 1, repr(expr)
                expr = expr[0]
            assert all(isinstance(x, scoretools.Component) for x in [expr])
            if any(isinstance(x, scoretools.GraceContainer) for x in [expr]):
                message = 'must attach grace container to note or chord.'
                raise Exception(message)
            old = self[i]
            selection = selectiontools.ContiguousSelection(old)
            spanners_receipt = selection._get_dominant_spanners()
            for child in iterate([old]).by_class():
                for spanner in child._get_spanners():
                    spanner._remove(child)
            if i < 0:
                i = len(self) + i
            del(self[i])
            # must withdraw from spanners before withdrawing from parentage!
            # otherwise begin / end assessments don't work!
            if withdraw_components_in_expr_from_crossing_spanners:
                selection = selectiontools.SliceSelection([expr])
                selection._withdraw_from_crossing_spanners()
            expr._set_parent(self)
            self._music.insert(i, expr)
            for spanner, index in spanners_receipt:
                spanner._insert(index, expr)
                expr._spanners.add(spanner)
        # slice assignment
        else:
            if isinstance(expr, str):
                expr = self._parse_string(expr)[:]
            elif isinstance(expr, list) and \
                len(expr) == 1 and \
                isinstance(expr[0], str):
                expr = self._parse_string(expr[0])[:]
            assert all(isinstance(x, scoretools.Component) for x in expr)
            if any(isinstance(x, scoretools.GraceContainer) for x in expr):
                message = 'must attach grace container to note or chord.'
                raise Exception(message)
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            spanners_receipt = self._get_spanners_that_dominate_slice(
                start, stop)
            for component in old:
                for child in iterate([component]).by_class():
                    for spanner in child._get_spanners():
                        spanner._remove(child)
            del(self[start:stop])
            # must withdraw before setting in self!
            # otherwise circular withdraw ensues!
            if withdraw_components_in_expr_from_crossing_spanners:
                selection = selectiontools.SliceSelection(expr)
                if selection._all_are_contiguous_components_in_same_logical_voice(
                    selection):
                    selection._withdraw_from_crossing_spanners()
            self._music.__setitem__(slice(start, start), expr)
            for component in expr:
                component._set_parent(self)
            for spanner, index in spanners_receipt:
                for component in reversed(expr):
                    spanner._insert(index, component)
                    component._spanners.add(spanner)
        for indicator in expr_indicators:
            if hasattr(indicator, '_update_effective_context'):
                indicator._update_effective_context()

    ### PUBLIC PROPERTIES ###

    @property
    def is_simultaneous(self):
        r'''Simultaneity status of container.

        ..  container:: example

            **Example 1.** Get simultaneity status of container:

            ::

                >>> container = Container()
                >>> container.append(Voice("c'8 d'8 e'8"))
                >>> container.append(Voice('g4.'))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                }

            ::

                >>> container.is_simultaneous
                False

        ..  container:: example

            **Example 2.** Set simultaneity status of container:

            ::

                >>> container = Container()
                >>> container.append(Voice("c'8 d'8 e'8"))
                >>> container.append(Voice('g4.'))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                }

            ::

                >>> container.is_simultaneous = True
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                <<
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                >>

        Returns boolean.
        '''
        return self._simultaneous

    @is_simultaneous.setter
    def is_simultaneous(self, expr):
        from abjad.tools import scoretools
        assert isinstance(expr, bool), repr(expr)
        if expr == True:
            assert all(
                isinstance(x, scoretools.Context) for x in self._music)
        self._simultaneous = expr
        self._update_later(offsets=True)

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_orphan_components(expr):
        from abjad.tools import scoretools
        for component in expr:
            if not isinstance(component, scoretools.Component):
                return False
            if not component._get_parentage().is_orphan:
                return False
        return True

    def _initialize_music(self, music):
        from abjad.tools import scoretools
        Selection = selectiontools.Selection
        if music is None:
            music = []
        if self._all_are_orphan_components(music):
            self._music = list(music)
            self[:]._set_parents(self)
        elif Selection._all_are_contiguous_components_in_same_logical_voice(
            music):
            music = selectiontools.SliceSelection(music)
            parent, start, stop = music._get_parent_and_start_stop_indices()
            self._music = list(music)
            self[:]._set_parents(self)
            assert parent is not None
            parent._music.insert(start, self)
            self._set_parent(parent)
        elif isinstance(music, str):
            parsed = self._parse_string(music)
            self._music = []
            self.is_simultaneous = parsed.is_simultaneous
            if parsed.is_simultaneous or \
                not Selection._all_are_contiguous_components_in_same_logical_voice(
                parsed[:]):
                while len(parsed):
                    self.append(parsed.pop(0))
            else:
                self[:] = parsed[:]
        else:
            message = 'can not initialize container from {!r}.'
            message = message.format((music))
            raise TypeError(message)

    def _is_one_of_my_first_leaves(self, leaf):
        return leaf in self._get_descendants_starting_with()

    def _is_one_of_my_last_leaves(self, leaf):
        return leaf in self._get_descendants_stopping_with()

    def _move_spanners_to_children(self):
        for spanner in self._get_spanners():
            i = spanner.index(self)
            spanner._components.__setitem__(slice(i, i + 1), self[:])
            for component in self:
                component._spanners.add(spanner)
            self._spanners.discard(spanner)
        return self

    def _parse_string(self, string):
        from abjad.tools import lilypondfiletools
        from abjad.tools import lilypondparsertools
        from abjad.tools import rhythmtreetools
        user_input = string.strip()
        if user_input.startswith('abj:'):
            parser = lilypondparsertools.ReducedLyParser()
            parsed = parser(user_input[4:])
            if parser._toplevel_component_count == 1:
                parsed = Container([parsed])
        elif user_input.startswith('rtm:'):
            parsed = rhythmtreetools.parse_rtm_syntax(user_input[4:])
        else:
            if not user_input.startswith('<<') or \
                not user_input.endswith('>>'):
                user_input = '{{ {} }}'.format(user_input)
            parsed = lilypondparsertools.LilyPondParser()(user_input)
            if isinstance(parsed, lilypondfiletools.LilyPondFile):
                parsed = Container(parsed[:])
            assert isinstance(parsed, Container)
        return parsed

    def _scale(self, multiplier):
        self._scale_contents(multiplier)

    def _split_at_index(self, i, fracture_spanners=False):
        r'''Splits container to the left of index `i`.

        Preserves tuplet multiplier when container is a tuplet.

        Preserves time signature denominator when container is a measure.

        Resizes resizable containers.

        Returns split parts.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import spannertools
        # partition my music
        left_music = self[:i]
        right_music = self[i:]
        # instantiate new left and right containers
        if isinstance(self, scoretools.Measure):
            time_signature = self._get_effective(
                indicatortools.TimeSignature)
            denominator = time_signature.denominator
            left_duration = sum([x._get_duration() for x in left_music])
            left_pair = mathtools.NonreducedFraction(left_duration)
            left_pair = left_pair.with_multiple_of_denominator(denominator)
            left_time_signature = indicatortools.TimeSignature(left_pair)
            left = type(self)(left_time_signature, left_music)
            right_duration = sum([x._get_duration() for x in right_music])
            right_pair = mathtools.NonreducedFraction(right_duration)
            right_pair = right_pair.with_multiple_of_denominator(denominator)
            right_time_signature = indicatortools.TimeSignature(right_pair)
            right = type(self)(right_time_signature, right_music)
        elif isinstance(self, scoretools.FixedDurationTuplet):
            multiplier = self.multiplier
            left = type(self)(1, left_music)
            right = type(self)(1, right_music)
            target_duration = multiplier * left._contents_duration
            left.target_duration = target_duration
            target_duration = multiplier * right._contents_duration
            right.target_duration = target_duration
        elif isinstance(self, scoretools.Tuplet):
            multiplier = self.multiplier
            left = type(self)(multiplier, left_music)
            right = type(self)(multiplier, right_music)
        else:
            left = type(self)(left_music)
            right = type(self)(right_music)
        # save left and right containers together for iteration
        halves = (left, right)
        nonempty_halves = [half for half in halves if len(half)]
        # give my attached spanners to my children
        self._move_spanners_to_children()
        # incorporate left and right parents in score if possible
        selection = selectiontools.SliceSelection(self)
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        if parent is not None:
            parent._music.__setitem__(slice(start, stop + 1), nonempty_halves)
            for part in nonempty_halves:
                part._set_parent(parent)
        else:
            left._set_parent(None)
            right._set_parent(None)
        # fracture spanners if requested
        if fracture_spanners:
            for spanner in left._get_spanners():
                index = spanner.index(left)
                spanner.fracture(index, direction=Right)
        # return new left and right containers
        return halves

    def _split_by_duration(
        self,
        duration,
        fracture_spanners=False,
        tie_split_notes=True,
        ):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # check input
        duration = durationtools.Duration(duration)
        assert 0 <= duration, repr(duration)
        # if zero duration then return empty list and self
        if duration == 0:
            return [], self
        # get split point score offset
        global_split_point = self._get_timespan().start_offset + duration
        # get any duration-crossing descendents
        cross_offset = self._get_timespan().start_offset + duration
        duration_crossing_descendants = []
        for descendant in self._get_descendants():
            start_offset = descendant._get_timespan().start_offset
            stop_offset = descendant._get_timespan().stop_offset
            if start_offset < cross_offset < stop_offset:
                duration_crossing_descendants.append(descendant)
        # get any duration-crossing measure descendents
        measures = [
            x for x in duration_crossing_descendants
            if isinstance(x, scoretools.Measure)
            ]
        # if we must split a power-of-two measure at non-power-of-two
        # split point then go ahead and transform the power-of-two measure
        # to non-power-of-two equivalent now;
        # code that crawls and splits later on will be happier
        if len(measures) == 1:
            measure = measures[0]
            split_point_in_measure = \
                global_split_point - measure._get_timespan().start_offset
            if measure.has_non_power_of_two_denominator:
                if not measure.implied_prolation ==\
                    split_point_in_measure.implied_prolation:
                    raise NotImplementedError
            elif not mathtools.is_nonnegative_integer_power_of_two(
                split_point_in_measure.denominator):
                non_power_of_two_factors = mathtools.remove_powers_of_two(
                    split_point_in_measure.denominator)
                non_power_of_two_factors = mathtools.factors(
                    non_power_of_two_factors)
                non_power_of_two_product = 1
                for non_power_of_two_factor in non_power_of_two_factors:
                    non_power_of_two_product *= non_power_of_two_factor
                scoretools.scale_measure_denominator_and_adjust_measure_contents(
                    measure, non_power_of_two_product)
                # rederive duration crosses with possibly new measure contents
                cross_offset = self._get_timespan().start_offset + duration
                duration_crossing_descendants = []
                for descendant in self._get_descendants():
                    start_offset = descendant._get_timespan().start_offset
                    stop_offset = descendant._get_timespan().stop_offset
                    if start_offset < cross_offset < stop_offset:
                        duration_crossing_descendants.append(descendant)
        elif 1 < len(measures):
            message = 'measures can not nest.'
            raise Exception(message)
        # any duration-crossing leaf will be at end of list
        bottom = duration_crossing_descendants[-1]
        did_split_leaf = False
        # if split point necessitates leaf split
        if isinstance(bottom, scoretools.Leaf):
            assert isinstance(bottom, scoretools.Leaf)
            did_split_leaf = True
            split_point_in_bottom = \
                global_split_point - bottom._get_timespan().start_offset
            left_list, right_list = bottom._split_by_duration(
                split_point_in_bottom,
                fracture_spanners=fracture_spanners,
                tie_split_notes=tie_split_notes,
                )
            right = right_list[0]
            leaf_right_of_split = right
            leaf_left_of_split = left_list[-1]
            duration_crossing_containers = duration_crossing_descendants[:-1]
            if not len(duration_crossing_containers):
                return left_list, right_list
        # if split point falls between leaves
        # then find leaf to immediate right of split point
        # in order to start upward crawl through duration-crossing containers
        else:
            duration_crossing_containers = duration_crossing_descendants[:]
            for leaf in iterate(bottom).by_class(scoretools.Leaf):
                if leaf._get_timespan().start_offset == global_split_point:
                    leaf_right_of_split = leaf
                    leaf_left_of_split = leaf_right_of_split._get_leaf(-1)
                    break
            else:
                message = 'can not split empty container {!r}.'
                message = message.format(bottom)
                raise Exception(message)
        # find component to right of split that is also immediate child of
        # last duration-crossing container
        for component in \
            leaf_right_of_split._get_parentage(include_self=True):
            if component._parent is duration_crossing_containers[-1]:
                highest_level_component_right_of_split = component
                break
        else:
            message = 'should we be able to get here?'
            raise ValueError(message)
        # crawl back up through duration-crossing containers and
        # fracture spanners if requested
        if fracture_spanners:
            start_offset = leaf_right_of_split._get_timespan().start_offset
            for parent in leaf_right_of_split._get_parentage():
                if parent._get_timespan().start_offset == start_offset:
                    for spanner in parent._get_spanners():
                        index = spanner.index(parent)
                        spanner.fracture(index, direction=Left)
                if parent is component:
                    break
        # crawl back up through duration-crossing containers and split each
        previous = highest_level_component_right_of_split
        for duration_crossing_container in \
            reversed(duration_crossing_containers):
            assert isinstance(
                duration_crossing_container, scoretools.Container)
            i = duration_crossing_container.index(previous)
            left, right = duration_crossing_container._split_at_index(
                i,
                fracture_spanners=fracture_spanners,
                )
            previous = right
        # NOTE: If logical tie here is convenience, then fusing is good.
        #       If logical tie here is user-given, then fusing is less good.
        #       Maybe later model difference between user logical ties and not.
        left_logical_tie = leaf_left_of_split._get_logical_tie()
        right_logical_tie = leaf_right_of_split._get_logical_tie()
        left_logical_tie._fuse_leaves_by_immediate_parent()
        right_logical_tie._fuse_leaves_by_immediate_parent()
        # reapply tie here if crawl above killed tie applied to leaves
        if did_split_leaf:
            if tie_split_notes and \
                isinstance(leaf_left_of_split, scoretools.Note):
                if leaf_left_of_split._get_parentage().root is \
                    leaf_right_of_split._get_parentage().root:
                    leaves_around_split = \
                        (leaf_left_of_split, leaf_right_of_split)
                    selection = selectiontools.ContiguousSelection(
                        leaves_around_split)
                    selection._attach_tie_spanner_to_leaf_pair()
        # return pair of left and right list-wrapped halves of container
        return ([left], [right])

    ### PUBLIC METHODS ###

    def append(self, component):
        r'''Appends `component` to container.

        ..  container:: example

            ::

                >>> container = Container("c'4 ( d'4 f'4 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                }

            ::

                >>> container.append(Note("e'4"))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

        Returns none.
        '''
        self.__setitem__(slice(len(self), len(self)), [component])

    def extend(self, expr):
        r'''Extends container with `expr`.

        ..  container:: example

            ::

                >>> container = Container("c'4 ( d'4 f'4 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                }

            ::

                >>> notes = [Note("e'32"), Note("d'32"), Note("e'16")]
                >>> container.extend(notes)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'32
                    d'32
                    e'16
                }

        Returns none.
        '''
        self.__setitem__(
            slice(len(self), len(self)),
            expr.__getitem__(slice(0, len(expr)))
            )

    def index(self, component):
        r'''Returns index of `component` in container.

        ..  container:: example

            ::

                >>> container = Container("c'4 d'4 f'4 e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4
                    d'4
                    f'4
                    e'4
                }

            ::

                >>> note = container[-1]
                >>> note
                Note("e'4")

            ::

                >>> container.index(note)
                3

        Returns nonnegative integer.
        '''
        for i, element in enumerate(self._music):
            if element is component:
                return i
        else:
            message = 'component {!r} not in Abjad container {!r}.'
            message = message.format(component, self)
            raise ValueError(message)

    def insert(self, i, component, fracture_spanners=False):
        r'''Inserts `component` at index `i` in container.

        ..  container:: example

            **Example 1.** Insert note. Do not fracture spanners:

            ::

                >>> container = Container([])
                >>> container.extend("fs16 cs' e' a'")
                >>> container.extend("cs''16 e'' cs'' a'")
                >>> container.extend("fs'16 e' cs' fs")
                >>> slur = spannertools.Slur()
                >>> attach(slur, container[:])
                >>> slur.direction = Down
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

            ::

                >>> container.insert(-4, Note("e'4"), fracture_spanners=False)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    e'4
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

        ..  container:: example

            **Example 2.** Insert note. Fracture spanners:

            ::

                >>> container = Container([])
                >>> container.extend("fs16 cs' e' a'")
                >>> container.extend("cs''16 e'' cs'' a'")
                >>> container.extend("fs'16 e' cs' fs")
                >>> slur = spannertools.Slur()
                >>> attach(slur, container[:])
                >>> slur.direction = Down
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

            ::

                >>> container.insert(-4, Note("e'4"), fracture_spanners=True)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16 )
                    e'4
                    fs'16 _ (
                    e'16
                    cs'16
                    fs16 )
                }

        Returns none.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        assert isinstance(component, scoretools.Component)
        assert isinstance(i, int)
        if not fracture_spanners:
            self.__setitem__(slice(i, i), [component])
            return
        component._set_parent(self)
        self._music.insert(i, component)
        previous_leaf = component._get_leaf(-1)
        if previous_leaf:
            for spanner in previous_leaf._get_spanners():
                index = spanner.index(previous_leaf)
                spanner.fracture(index, direction=Right)
        next_leaf = component._get_leaf(1)
        if next_leaf:
            for spanner in next_leaf._get_spanners():
                index = spanner.index(next_leaf)
                spanner.fracture(index, direction=Left)

    def pop(self, i=-1):
        r'''Pops component from container at index `i`.

        ..  container:: example

            ::

                >>> container = Container("c'4 ( d'4 f'4 ) e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

            ::

                >>> container.pop()
                Note("e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                }

        Returns component.
        '''
        component = self[i]
        del(self[i])
        return component

    def remove(self, component):
        r'''Removes `component` from container.

        ..  container:: example

            ::

                >>> container = Container("c'4 ( d'4 f'4 ) e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

            ::

                >>> note = container[2]
                >>> note
                Note("f'4")

            ::

                >>> container.remove(note)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'4 (
                    d'4 )
                    e'4
                }

        Returns none.
        '''
        i = self.index(component)
        del(self[i])

    def reverse(self):
        r'''Reverses contents of container.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 [ d'8 ] e'8 ( f'8 )")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 [
                    d'8 ]
                    e'8 (
                    f'8 )
                }

            ::

                >>> staff.reverse()
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff) # doctest: +SKIP
                \new Staff {
                    f'8 (
                    e'8 )
                    d'8 [
                    c'8 ]
                }

        Returns none.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        def _offset(x, y):
            if x._get_timespan().start_offset < y._get_timespan().start_offset:
                return -1
            elif y._get_timespan().start_offset < x._get_timespan().start_offset:
                return 1
            else:
                return 0
        self._music.reverse()
        self._update_later(offsets=True)
        spanners = self._get_descendants()._get_spanners()
        for s in spanners:
            s._components.sort(_offset)

    def select_leaves(
        self,
        start=0,
        stop=None,
        leaf_classes=None,
        recurse=True,
        allow_discontiguous_leaves=False,
        ):
        r'''Selects leaves in container.

        ..  container:: example

            ::

                >>> container = Container("c'8 d'8 r8 e'8")

            ::

                >>> container.select_leaves()
                ContiguousSelection(Note("c'8"), Note("d'8"), Rest('r8'), Note("e'8"))

        Returns contiguous leaf selection or free leaf selection.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        Selection = selectiontools.Selection
        leaf_classes = leaf_classes or (scoretools.Leaf,)
        expr = self
        if recurse:
            expr = iterate(expr).by_class(scoretools.Leaf)
        music = [
            component for component in expr
            if isinstance(component, leaf_classes)
            ]
        music = music[start:stop]
        if allow_discontiguous_leaves:
            selection = selectiontools.Selection(music=music)
        else:
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                music)
            selection = selectiontools.ContiguousSelection(music=music)
        return selection

    def select_notes_and_chords(self):
        r'''Selects notes and chords in container.

        ..  container:: example

            ::

                >>> container.select_notes_and_chords()
                Selection(Note("c'8"), Note("d'8"), Note("e'8"))

        Returns leaf selection.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        generator = iterate(self).by_class((scoretools.Note, scoretools.Chord))
        return selectiontools.Selection(generator)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.markuptools.MarkupCommand import MarkupCommand


def test_markuptools_MarkupCommand___repr___01():
    r'''Repr is evaluable.
    '''

    markup_command_1 = markuptools.MarkupCommand('hspace', 0)
    markup_command_2 = eval(repr(markup_command_1))

    assert markup_command_1 is not markup_command_2
    assert markup_command_1 == markup_command_2



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_01():
    r'''Make train of 1/16th notes equal to 1/4 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(0, Duration(1, 16), Duration(1, 4)))

    r'''
    \new Voice {
        c'16
        c'16
        c'16
        c'16
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
        }
        '''
        )


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_02():
    r'''Make train of 1/16th notes equal to 9/32 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(0, Duration(1, 16), Duration(9, 32)))

    r'''
    \new Voice {
        c'16
        c'16
        c'16
        c'16
        c'32
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'32
        }
        '''
        )


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_03():
    r'''Make train of 1/16th notes equal to only 1/128 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(0, Duration(1, 16), Duration(1, 128)))

    r'''
    \new Voice {
        c'128
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'128
        }
        '''
        )


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_04():
    r'''Make train of 1/16th notes equal to 4/10 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(0, Duration(1, 16), Duration(4, 10)))

    r'''
    \new Voice {
        c'16
        c'16
        c'16
        c'16
        c'16
        c'16
        \times 4/5 {
            c'32
        }
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'16
            c'16
            \times 4/5 {
                c'32
            }
        }
        '''
        )


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_05():
    r'''Make train of written 1/16th notes within measure of 5/18.
    '''

    measure = Measure((5, 18), scoretools.make_repeated_notes_with_shorter_notes_at_end(
        0, Duration(1, 16), Duration(5, 18), prolation = Duration(16, 18)))

    r'''
    {
        \time 5/18
        \scaleDurations #'(8 . 9) {
            c'16
            c'16
            c'16
            c'16
            c'16
        }
    }
    '''

    assert inspect(measure).is_well_formed()
    assert systemtools.TestManager.compare(
        measure,
        r'''
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'16
                c'16
                c'16
                c'16
                c'16
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools import TreeNode
from abjad.tools.documentationtools.GraphvizObject import GraphvizObject


class GraphvizNode(TreeNode, GraphvizObject):
    r'''A Graphviz node.
    '''

    ### INITIALIZER ###

    def __init__(self, attributes=None, name=None):
        TreeNode.__init__(self, name=name)
        GraphvizObject.__init__(self, attributes=attributes)
        self._edges = set([])

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        node_def = self._format_value(self.canonical_name)
        if len(self.attributes):
            result = []
            result.extend(self._format_attribute_list(self.attributes))
            result[0] = '{} {}'.format(node_def, result[0])
            return result
        return [node_def + ';']

    ### PUBLIC PROPERTIES ###

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz node.

        Returns string.
        '''
        if self.name is not None:
            return self.name
        return 'node_' + '_'.join(str(x) for x in self.graph_order)

    @property
    def edges(self):
        r'''Edges of Graphviz node.

        Returns tuple.
        '''
        return tuple(self._edges)



# -*- encoding: utf-8 -*-


def iterate_sequence_nwise_cyclic(sequence, n):
    '''Iterate elements in `sequence` cyclically `n` at a time:

    ::

        >>> g = sequencetools.iterate_sequence_nwise_cyclic(range(6), 3)
        >>> for n in range(10):
        ...   print g.next()
        (0, 1, 2)
        (1, 2, 3)
        (2, 3, 4)
        (3, 4, 5)
        (4, 5, 0)
        (5, 0, 1)
        (0, 1, 2)
        (1, 2, 3)
        (2, 3, 4)
        (3, 4, 5)

    Returns generator.
    '''

    element_buffer = []
    long_enough = False
    for element in sequence:
        element_buffer.append(element)
        if not long_enough:
            if n <= len(element_buffer):
                long_enough = True
        if long_enough:
            yield tuple(element_buffer[-n:])

    len_sequence = len(element_buffer)
    current = len_sequence - n + 1
    while True:
        output = []
        for local_offset in range(n):
            index = (current + local_offset) % len_sequence
            output.append(element_buffer[index])
        yield tuple(output)
        current += 1
        current %= len_sequence



# -*- encoding: utf-8 -*-
from abjad.tools.markuptools.Markup import Markup
from abjad.tools.datastructuretools.TypedList import TypedList


class MarkupInventory(TypedList):
    '''Abjad model of an ordered list of markup:

    ::

        >>> inventory = markuptools.MarkupInventory(['foo', 'bar'])

    ::

        >>> inventory
        MarkupInventory([Markup(('foo',)), Markup(('bar',))])

    Markup inventories implement the list interface and are mutable.
    '''

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        return Markup



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LayoutBlock_context_blocks_01():

    layout_block = lilypondfiletools.LayoutBlock()
    context_block = lilypondfiletools.ContextBlock('Score')
    override(context_block).bar_number.transparent = True
    override(context_block).time_signature.break_visibility = \
        schemetools.Scheme('end-of-line-invisible')
    layout_block.context_blocks.append(context_block)

    assert systemtools.TestManager.compare(
        layout_block,
        r'''
        \layout {
            \context {
                \Score
                \override BarNumber #'transparent = ##t
                \override TimeSignature #'break-visibility = #end-of-line-invisible
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class Slur(Spanner):
    r'''A slur.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> slur = spannertools.Slur()
        >>> attach(slur, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 (
            d'8
            e'8
            f'8 )
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self,
        components=None,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            components,
            overrides=overrides,
            )
        self.direction = direction

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.direction = self.direction

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            direction = self.direction
            if direction is not None:
                result.append('{} ('.format(
                    stringtools.arg_to_tridirectional_lilypond_symbol(
                        direction)))
            else:
                result.append('(')
        if self._is_my_last_leaf(leaf):
            result.append(')')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets and sets direction of slur.

        Returns up, down or none.
        '''
        return self._direction

    @direction.setter
    def direction(self, arg):
        self._direction = \
            stringtools.arg_to_tridirectional_lilypond_symbol(arg)



# -*- encoding: utf-8 -*-
from abjad.tools.lilypondfiletools.NonattributedBlock \
	import NonattributedBlock


class BookBlock(NonattributedBlock):
    r'''Abjad model of LilyPond input file book block:

    ::

        >>> book_block = lilypondfiletools.BookBlock()

    ::

        >>> book_block
        BookBlock()

    ..  doctest::

        >>> print format(book_block)
        \book {}

    Returns book block.
    '''

    def __init__(self):
        NonattributedBlock.__init__(self)
        self._escaped_name = r'\book'
        self._is_formatted_when_empty = True



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchRangeInventory_name_01():

    inventory = pitchtools.PitchRangeInventory(['[A0, C8]'])
    assert inventory.custom_identifier is None

    inventory.custom_identifier = 'blue inventory'
    assert inventory.custom_identifier == 'blue inventory'


def test_pitchtools_PitchRangeInventory_name_02():

    inventory = pitchtools.PitchRangeInventory(['[A0, C8]'])
    assert pytest.raises(Exception, 'inventory.custom_identifier = 99')



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_by_counts_01():
    r'''Partition sequence by positive counts.
    '''

    parts = sequencetools.partition_sequence_by_counts(range(16), [4, 6], cyclic=True, overhang=True)
    assert parts == [[0, 1, 2, 3], [4, 5, 6, 7, 8, 9], [10, 11, 12, 13], [14, 15]]


def test_sequencetools_partition_sequence_by_counts_02():
    r'''Partition sequence by nonnegative counts.
    '''

    counts = (0, 2, 0, 0, 4)
    parts = sequencetools.partition_sequence_by_counts(range(10), counts, cyclic=True, overhang=True)
    assert parts == [[], [0, 1], [], [], [2, 3, 4, 5], [], [6, 7], [], [], [8, 9]]



def test_sequencetools_partition_sequence_by_counts_03():
    r'''Partition sequence by positive counts.
    '''

    parts = sequencetools.partition_sequence_by_counts(range(16), [4, 6], cyclic=True, overhang=False)
    assert parts == [[0, 1, 2, 3], [4, 5, 6, 7, 8, 9], [10, 11, 12, 13]]


def test_sequencetools_partition_sequence_by_counts_04():
    r'''Partition sequence by nonnegative counts.
    '''

    counts = (0, 2, 0, 0, 4)
    parts = sequencetools.partition_sequence_by_counts(range(10), counts, cyclic=True, overhang=False)
    assert parts == [[], [0, 1], [], [], [2, 3, 4, 5], [], [6, 7], [], []]



def test_sequencetools_partition_sequence_by_counts_05():
    r'''Partition sequence by positive counts.
    '''

    parts = sequencetools.partition_sequence_by_counts(range(16), [4, 6], cyclic=False, overhang=True)
    assert parts == [[0, 1, 2, 3], [4, 5, 6, 7, 8, 9], [10, 11, 12, 13, 14, 15]]


def test_sequencetools_partition_sequence_by_counts_06():
    r'''Partition sequence by nonnegative counts.
    '''

    counts = (0, 2, 0, 0, 4)
    parts = sequencetools.partition_sequence_by_counts(range(10), counts, cyclic=False, overhang=True)
    assert parts == [[], [0, 1], [], [], [2, 3, 4, 5], [6, 7, 8, 9]]


def test_sequencetools_partition_sequence_by_counts_07():
    r'''Partition list by positive counts.
    '''

    parts = sequencetools.partition_sequence_by_counts(range(16), [4, 6], cyclic=False, overhang=False)
    assert parts == [[0, 1, 2, 3], [4, 5, 6, 7, 8, 9]]


def test_sequencetools_partition_sequence_by_counts_08():
    r'''Partition list by nonnegative counts.
    '''

    counts = (0, 2, 0, 0, 4)
    parts = sequencetools.partition_sequence_by_counts(range(10), counts, cyclic=False, overhang=False)
    assert parts == [[], [0, 1], [], [], [2, 3, 4, 5]]


def test_sequencetools_partition_sequence_by_counts_09():
    r'''Partition Abjad container.
    '''

    container = Container("c'8 d'8 e'8 f'8 g'8 a'8")
    parts = sequencetools.partition_sequence_by_counts(container, [1, 2], cyclic=False, overhang=False)

    "[{c'8}, {d'8, e'8}]"

    assert len(parts) == 2
    assert format(parts[0]) == format(Container("c'8"))
    assert parts[0][0] is not container[0]
    assert format(parts[1]) == format(Container("d'8 e'8"))
    assert parts[1][0] is not container[1]
    assert parts[1][1] is not container[2]


def test_sequencetools_partition_sequence_by_counts_10():
    r'''Partition list of arbitrary objects instead of just integers.
    '''

    parts = sequencetools.partition_sequence_by_counts('this is text', [1, 3], cyclic=True, overhang=True)
    assert parts == ['t', 'his', ' ', 'is ', 't', 'ext']



# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_pitch_class_numbers(
    expr, 
    number=True, 
    color=False,
    markup_direction=Down,
    ):
    r'''Label leaves in `expr` with pitch-class numbers:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_pitch_class_numbers(staff)
        >>> print format(staff)
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 2 }
            e'8 _ \markup { \small 4 }
            f'8 _ \markup { \small 5 }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    When ``color=True`` call
    :func:`~abjad.tools.labeltools.color_note_head_by_numbered_pitch_class_color_map`:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_pitch_class_numbers(
        ...     staff, color=True, number=False)
        >>> print format(staff)
        \new Staff {
            \once \override NoteHead #'color = #(x11-color 'red)
            c'8
            \once \override NoteHead #'color = #(x11-color 'orange)
            d'8
            \once \override NoteHead #'color = #(x11-color 'ForestGreen)
            e'8
            \once \override NoteHead #'color = #(x11-color 'MediumOrchid)
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    You can set `number` and `color` at the same time.

    Returns none.
    '''
    from abjad.tools import labeltools

    for note in iterate(expr).by_class(scoretools.Note):
        if number:
            label = markuptools.MarkupCommand(
                'small', str(abs(note.written_pitch.numbered_pitch_class)))
            markup = markuptools.Markup(label, markup_direction)
            attach(markup, note)
        if color:
            labeltools.color_note_head_by_numbered_pitch_class_color_map(note)



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_markuptools_MarkupCommand___setattr___01():

    command = markuptools.MarkupCommand('draw-circle', 1, 0.1, False)
    assert pytest.raises(AttributeError, "command.foo = 'bar'")



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override
from abjad.tools.datastructuretools.TypedList import TypedList


class ClefInventory(TypedList):
    '''An ordered list of clefs.

    ::

        >>> inventory = indicatortools.ClefInventory(['treble', 'bass'])

    ::

        >>> inventory
        ClefInventory([Clef('treble'), Clef('bass')])

    ::

        >>> 'treble' in inventory
        True

    ::

        >>> Clef('treble') in inventory
        True

    ::

        >>> 'alto' in inventory
        False

    ::

        >>> show(inventory) # doctest: +SKIP

    Clef inventories implement the list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r'''Illustrates clef inventory.

        ::

            >>> show(inventory) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        staff = scoretools.Staff()
        for clef in self:
            rest = scoretools.Rest((1, 8))
            clef = copy.copy(clef)
            attach(clef, rest)
            staff.append(rest)
        override(staff).clef.full_size_change = True
        override(staff).rest.transparent = True
        override(staff).time_signature.stencil = False
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(staff)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        from abjad.tools import indicatortools
        return indicatortools.Clef

    @property
    def _one_line_menuing_summary(self):
        return ', '.join([clef.name for clef in self])



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class HiddenStaffSpanner(Spanner):
    r'''A hidden staff spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> spanner = spannertools.HiddenStaffSpanner()
        >>> attach(spanner, staff[:2])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \stopStaff
            c'8
            d'8
            \startStaff
            e'8
            f'8
        }

    Hide staff behind leaves in spanner.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
        return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_pair_duration_sequence_elements_with_input_pair_values_01():

    duration_sequence = [10, 10, 10, 10]

    result = sequencetools.pair_duration_sequence_elements_with_input_pair_values(
        duration_sequence, [('red', 100)])
    assert result == [(10, 'red'), (10, 'red'), (10, 'red'), (10, 'red')]

    result = sequencetools.pair_duration_sequence_elements_with_input_pair_values(
        duration_sequence, [('red', 1), ('blue', 99)])

    result = sequencetools.pair_duration_sequence_elements_with_input_pair_values(
        duration_sequence, [('red', 1), ('blue', 1), ('green', 98)])
    assert result == [(10, 'red'), (10, 'green'), (10, 'green'), (10, 'green')]

    result = sequencetools.pair_duration_sequence_elements_with_input_pair_values(
        duration_sequence, [('red', 100), ('blue', 100), ('green', 100)])
    assert result == [(10, 'red'), (10, 'red'), (10, 'red'), (10, 'red')]



# -*- encoding: utf-8 -*-
import os
import re
from abjad.tools import abctools


class PersistenceAgent(abctools.AbjadObject):
    r'''A wrapper around the Abjad persistence methods.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> persist(staff)
            PersistenceAgent(client={c'4, e'4, d'4, f'4})

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        self._client = client

    ### PRIVATE PROPERTIES ###

    @property
    def client(self):
        r'''Client of persistence agent.

        Returns selection or component.
        '''
        return self._client

    ### PUBLIC METHODS ###

    def as_ly(self, ly_filename=None):
        r'''Persists client as LilyPond file.

        If `ly_filename` is none, a file path will be autogenerated.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_ly('~/example.ly'): # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/Desktop/test.ly'
            0.04491996765136719

        Returns output path and elapsed formatting time.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        # get the illustration
        assert '__illustrate__' in dir(self._client)
        illustration = self._client.__illustrate__()
        # validate the output path
        if ly_filename is None:
            ly_filename = systemtools.IOManager.get_next_output_file_name()
            ly_filepath = os.path.join(
                abjad_configuration.abjad_output_directory_path,
                ly_filename,
                )
        else:
            ly_filepath = os.path.expanduser(ly_filename)
        assert ly_filepath.endswith('.ly'), ly_filepath
        # format the illustration
        timer = systemtools.Timer()
        with timer:
            lilypond_format = format(illustration, 'lilypond')
        abjad_formatting_time = timer.elapsed_time
        # write the formatted illustration to disk
        systemtools.IOManager.ensure_directory_existence(
            os.path.dirname(ly_filepath))
        with open(ly_filepath, 'w') as file_handle:
            file_handle.write(lilypond_format)
        return ly_filepath, abjad_formatting_time

    def as_midi(self, midi_filename=None):
        r'''Persists client as MIDI file.

        If `midi_filename` is none, a file path will be autogenerated.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_midi(): # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/.abjad/output/1415.midi'
            0.07831692695617676
            1.0882699489593506

        Returns output path, elapsed formatting time and elapsed rendering
        time.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import systemtools
        assert '__illustrate__' in dir(self._client)
        illustration = self._client.__illustrate__()
        assert hasattr(illustration, 'score_block')
        illustration.score_block.append(lilypondfiletools.MIDIBlock())
        if midi_filename is not None:
            midi_filepath = os.path.expanduser(midi_filename)
            ly_filepath = '{}.ly'.format(os.path.splitext(midi_filepath)[0])
        else:
            ly_filepath = None
        ly_filepath, abjad_formatting_time = type(self)(illustration).as_ly(
            ly_filepath)
        timer = systemtools.Timer()
        with timer:
            systemtools.IOManager.run_lilypond(ly_filepath)
        lilypond_rendering_time = timer.elapsed_time
        if os.name == 'nt':
            extension = 'mid'
        else:
            extension = 'midi'
        midi_filepath = '{}.{}'.format(
            os.path.splitext(ly_filepath)[0],
            extension,
            )
        return midi_filepath, abjad_formatting_time, lilypond_rendering_time

    def as_module(self, module_filename, object_name):
        r'''Persists client as Python module.

        ::

            >>> inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 1),
            ...     timespantools.Timespan(2, 4),
            ...     timespantools.Timespan(6, 8),
            ...     ])
            >>> persist(inventory).as_module( # doctest: +SKIP
            ...     '~/example.py', 'inventory')

        Returns none.
        '''
        from abjad.tools import systemtools
        assert '_storage_format' in dir(self._client)
        result = ['# -*- encoding: utf-8 -*-']
        storage_pieces = format(self._client, 'storage').splitlines()
        pattern = re.compile(r'\b[a-z]+tools\b')
        tools_package_names = set()
        for line in storage_pieces:
            match = pattern.search(line)
            while match is not None:
                group = match.group()
                tools_package_names.add(group)
                end = match.end()
                match = pattern.search(line, pos=end)
        for name in sorted(tools_package_names):
            result.append('from abjad.tools import {}'.format(name))
        result.append('')
        result.append('')
        result.append('{} = {}'.format(object_name, storage_pieces[0]))
        result.extend(storage_pieces[1:])
        result = '\n'.join(result)
        module_filepath = os.path.expanduser(module_filename)
        systemtools.IOManager.ensure_directory_existence(
            os.path.dirname(module_filepath))
        with open(module_filepath, 'w') as f:
            f.write(result)

    def as_pdf(self, pdf_filename=None):
        r'''Persists client as PDF.

        If `pdf_filename` is none, a file path will be autogenerated.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_pdf(): # doctest: +SKIP
            '/Users/josiah/.abjad/output/1416.pdf'
            0.047142982482910156
            0.7839350700378418

        Returns output path, elapsed formatting time and elapsed rendering
        time.
        '''
        from abjad.tools import systemtools
        assert '__illustrate__' in dir(self._client)
        # validate the output path
        if pdf_filename is not None:
            pdf_filepath = os.path.expanduser(pdf_filename)
            ly_filepath = '{}.ly'.format(os.path.splitext(pdf_filepath)[0])
        else:
            ly_filepath = None
        # format and write the lilypond file
        ly_filepath, abjad_formatting_time = self.as_ly(ly_filepath)
        pdf_filepath = '{}.pdf'.format(os.path.splitext(ly_filepath)[0])
        # render the pdf
        timer = systemtools.Timer()
        with timer:
            systemtools.IOManager.run_lilypond(ly_filepath)
        lilypond_rendering_time = timer.elapsed_time
        return pdf_filepath, abjad_formatting_time, lilypond_rendering_time



# -*- encoding: utf-8 -*-


def is_null_tuple(expr):
    r'''True when `expr` is a tuple of length ``0``:

    ::

        >>> sequencetools.is_null_tuple(())
        True

    Otherwise false:

    ::

        >>> sequencetools.is_null_tuple((19, 20, 21))
        False

    Returns boolean.
    '''

    return isinstance(expr, tuple) and not len(expr)



def contextualize(expr):
    r'''Applies LilyPond context setting to `expr`.

    Returns LilyPond context setting manager.
    '''
    from abjad.tools import lilypondnametools

    if getattr(expr, '_set', None) is None:
    #if not hasattr(expr, '_set'):
        manager = lilypondnametools.LilyPondSettingNameManager()
        expr._set = manager

    return expr._set



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class NoteHeadInventory(TypedList):
    r'''An ordered list of note heads.

    ::

        >>> chord = Chord([0, 1, 4], (1, 4))
        >>> inventory = scoretools.NoteHeadInventory(
        ...     client=chord,
        ...     tokens=[11, 10, 9],
        ...     )

    ::

        >>> print format(inventory)
        scoretools.NoteHeadInventory(
            [
                scoretools.NoteHead(
                    written_pitch=pitchtools.NamedPitch("a'"),
                    is_cautionary=False,
                    is_forced=False,
                    ),
                scoretools.NoteHead(
                    written_pitch=pitchtools.NamedPitch("bf'"),
                    is_cautionary=False,
                    is_forced=False,
                    ),
                scoretools.NoteHead(
                    written_pitch=pitchtools.NamedPitch("b'"),
                    is_cautionary=False,
                    is_forced=False,
                    ),
                ]
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        tokens=None,
        client=None,
        ):
        from abjad.tools import scoretools
        self._client = client
        TypedList.__init__(
            self,
            item_class=scoretools.NoteHead,
            keep_sorted=True,
            tokens=tokens,
            )

    ### PRIVATE METHODS ###

    def _on_insertion(self, item):
        item._client = self.client

    def _on_removal(self, item):
        item._client = None

    ### PRIVATE PROPERTIES ##

    @property
    def _item_callable(self):
        from abjad.tools import scoretools
        def coerce(token):
            if not isinstance(token, scoretools.NoteHead):
                token = scoretools.NoteHead(
                    written_pitch=token,
                    )
                token._client = self.client
            return token
        return coerce

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=(
                self._collection,
                )
            )

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''The note head inventory's chord client.
        '''
        return self._client

    ### PUBLIC METHODS ###

    def get(self, pitch):
        r'''Gets note head in note head inventory by `pitch`.

        ..  container:: example

            **Example 1.** Gets note head by pitch name:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> note_head = chord.note_heads.get("e'")
                >>> note_head.tweak.color = 'red'
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print format(chord)
                <
                    \tweak #'color #red
                    e'
                    cs''
                    f''
                >4

        ..  container:: example

            **Example 2.** Gets note head by pitch number:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> note_head = chord.note_heads.get(4)
                >>> note_head.tweak.color = 'red'
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print format(chord)
                <
                    \tweak #'color #red
                    e'
                    cs''
                    f''
                >4

        Raises missing note head error when chord contains no
        note head with `pitch`.

        Raises extra note head error when chord contains more than
        one note head with `pitch`.

        Returns note head.
        '''
        from abjad.tools import pitchtools
        result = []
        pitch = pitchtools.NamedPitch(pitch)
        for note_head in self:
            if note_head.written_pitch == pitch:
                result.append(note_head)
        count = len(result)
        if count == 0:
            message = 'missing note head.'
            raise ValueError(message)
        elif count == 1:
            note_head = result[0]
            return note_head
        else:
            message = 'extra note head.'
            raise ValueError(message)



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.ComplexBeam import ComplexBeam
from abjad.tools import durationtools


class MeasuredComplexBeam(ComplexBeam):
    r'''A measured complex beam spanner.

    ::

        >>> staff = Staff()
        >>> staff.append(Measure((2, 16), "c'16 d'16"))
        >>> staff.append(Measure((2, 16), "e'16 f'16"))
        >>> show(staff) # doctest: +SKIP

    ::

        >>> beam = spannertools.MeasuredComplexBeam()
        >>> attach(beam, staff.select_leaves())
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            {
                \time 2/16
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                d'16
            }
            {
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                e'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                f'16 ]
            }
        }

    Beams leaves in spanner explicitly.

    Groups leaves by measures.

    Formats top-level `span` beam between measures.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        lone=False, 
        span=1, 
        direction=None,
        overrides=None,
        ):
        ComplexBeam.__init__(
            self, 
            components=components, 
            lone=lone, 
            direction=direction,
            overrides=overrides,
            )
        self.span = span

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        ComplexBeam._copy_keyword_args(self, new)
        new.span = self.span

    def _format_before_leaf(self, leaf):
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        result = []
        left, right = None, None
        #if leaf.beam.beamable:
        if self.is_beamable_component(leaf):
            if self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            elif leaf._get_parentage(include_self=False).get_first(
                scoretools.Measure) is not None:
                measure = leaf._get_parentage(include_self=False).get_first(
                    scoretools.Measure)
                # leaf at beginning of measure
                if measure._is_one_of_my_first_leaves(leaf):
                    assert isinstance(self.span, int)
                    left = self.span
                    right = leaf.written_duration.flag_count
                # leaf at end of measure
                elif measure._is_one_of_my_last_leaves(leaf):
                    assert isinstance(self.span, int)
                    left = leaf.written_duration.flag_count
                    right = self.span
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def span(self):
        r'''Get top-level beam count:

        ::

            >>> staff = Staff()
            >>> staff.append(Measure((2, 16), "c'16 d'16"))
            >>> staff.append(Measure((2, 16), "e'16 f'16"))
            >>> beam = spannertools.MeasuredComplexBeam()
            >>> attach(beam, staff.select_leaves())
            >>> beam.span
            1

        Set top-level beam count:

        ::

            >>> staff = Staff()
            >>> staff.append(Measure((2, 16), "c'16 d'16"))
            >>> staff.append(Measure((2, 16), "e'16 f'16"))
            >>> beam = spannertools.MeasuredComplexBeam()
            >>> attach(beam, staff.select_leaves())
            >>> beam.span = 2
            >>> beam.span
            2

        Set nonnegative integer.
        '''
        return self._span

    @span.setter
    def span(self, arg):
        assert isinstance(arg, (int, type(None)))
        self._span = arg
